@@ -22,26 +22,16 @@
 
 #include <glib/gi18n.h>
 #include "bookmarks-export-druid-konqueror.h"
-#include "gul-gobject-misc.h"
 #include "gul-string.h"
 #include "gul-general.h"
 
 #include "xbel.h"
 
-/**
- * Private data
- */
-struct _GbExportDruidKonquerorPrivate {
-
-};
-
 static GHashTable *strings = NULL;
 
 /**
  * Private functions, only availble from this file
  */
-static void		gb_export_druid_konqueror_class_init		(GbExportDruidKonquerorClass *klass);
-static void		gb_export_druid_konqueror_init			(GbExportDruidKonqueror *e);
 static void		gb_export_druid_konqueror_finalize_impl		(GObject *o);
 static void		gb_export_druid_konqueror_init_strings 		(void);
 static const char *	gb_export_druid_konqueror_get_string_impl	(GbExportDruid *d, 
@@ -49,15 +39,12 @@ static const char *	gb_export_druid_konqueror_get_string_impl	(GbExportDruid *d,
 static GSList *		gb_export_druid_konqueror_get_locations_impl	(GbExportDruid *d);
 static GbIO *		gb_export_druid_konqueror_get_io_impl		(GbExportDruid *d);
 
-static gpointer gb_export_druid_class;
 
 /**
  * ExportDruidKonqueror object
  */
 
-MAKE_GET_TYPE (gb_export_druid_konqueror, "GbExportDruidKonqueror", GbExportDruidKonqueror, 
-	       gb_export_druid_konqueror_class_init, 
-	       gb_export_druid_konqueror_init, GB_TYPE_EXPORT_DRUID);
+G_DEFINE_TYPE (GbExportDruidKonqueror, gb_export_druid_konqueror, GB_TYPE_EXPORT_DRUID);
 
 static void
 gb_export_druid_konqueror_class_init (GbExportDruidKonquerorClass *klass)
@@ -66,27 +53,17 @@ gb_export_druid_konqueror_class_init (GbExportDruidKonquerorClass *klass)
 	GB_EXPORT_DRUID_CLASS (klass)->get_io = gb_export_druid_konqueror_get_io_impl;
 	GB_EXPORT_DRUID_CLASS (klass)->get_string = gb_export_druid_konqueror_get_string_impl;
 	G_OBJECT_CLASS (klass)->finalize = gb_export_druid_konqueror_finalize_impl;
-	gb_export_druid_class = g_type_class_peek_parent (klass);
 }
 
 static void 
 gb_export_druid_konqueror_init (GbExportDruidKonqueror *e)
 {
-	GbExportDruidKonquerorPrivate *p = g_new0 (GbExportDruidKonquerorPrivate, 1);
-	e->priv = p;
-	
 }
 
 static void
 gb_export_druid_konqueror_finalize_impl (GObject *o)
 {
-	GbExportDruidKonqueror *e = GB_EXPORT_DRUID_KONQUEROR (o);
-	GbExportDruidKonquerorPrivate *p = e->priv;
-
-	g_free (p);
-	
-	G_OBJECT_CLASS (gb_export_druid_class)->finalize (o);
-
+	G_OBJECT_CLASS (gb_export_druid_konqueror_parent_class)->finalize (o);
 }
 
 GbExportDruidKonqueror *
