@@ -22,26 +22,17 @@
 
 #include <glib/gi18n.h>
 #include "bookmarks-export-druid-netscape.h"
-#include "gul-gobject-misc.h"
 #include "gul-string.h"
 #include "gul-general.h"
 
 #include "bookmarks-netscape.h"
 
-/**
- * Private data
- */
-struct _GbExportDruidNetscapePrivate {
-
-};
 
 static GHashTable *strings = NULL;
 
 /**
  * Private functions, only availble from this file
  */
-static void		gb_export_druid_netscape_class_init		(GbExportDruidNetscapeClass *klass);
-static void		gb_export_druid_netscape_init			(GbExportDruidNetscape *e);
 static void		gb_export_druid_netscape_finalize_impl		(GObject *o);
 static void		gb_export_druid_netscape_init_strings 		(void);
 static const char *	gb_export_druid_netscape_get_string_impl	(GbExportDruid *d, 
@@ -49,15 +40,12 @@ static const char *	gb_export_druid_netscape_get_string_impl	(GbExportDruid *d,
 static GSList *		gb_export_druid_netscape_get_locations_impl	(GbExportDruid *d);
 static GbIO *		gb_export_druid_netscape_get_io_impl		(GbExportDruid *d);
 
-static gpointer gb_export_druid_class;
 
 /**
  * ExportDruidNetscape object
  */
 
-MAKE_GET_TYPE (gb_export_druid_netscape, "GbExportDruidNetscape", GbExportDruidNetscape, 
-	       gb_export_druid_netscape_class_init, 
-	       gb_export_druid_netscape_init, GB_TYPE_EXPORT_DRUID);
+G_DEFINE_TYPE (GbExportDruidNetscape, gb_export_druid_netscape, GB_TYPE_EXPORT_DRUID);
 
 static void
 gb_export_druid_netscape_class_init (GbExportDruidNetscapeClass *klass)
@@ -66,27 +54,17 @@ gb_export_druid_netscape_class_init (GbExportDruidNetscapeClass *klass)
 	GB_EXPORT_DRUID_CLASS (klass)->get_io = gb_export_druid_netscape_get_io_impl;
 	GB_EXPORT_DRUID_CLASS (klass)->get_string = gb_export_druid_netscape_get_string_impl;
 	G_OBJECT_CLASS (klass)->finalize = gb_export_druid_netscape_finalize_impl;
-	gb_export_druid_class = g_type_class_peek_parent (klass);
 }
 
 static void 
 gb_export_druid_netscape_init (GbExportDruidNetscape *e)
 {
-	GbExportDruidNetscapePrivate *p = g_new0 (GbExportDruidNetscapePrivate, 1);
-	e->priv = p;
-	
 }
 
 static void
 gb_export_druid_netscape_finalize_impl (GObject *o)
 {
-	GbExportDruidNetscape *e = GB_EXPORT_DRUID_NETSCAPE (o);
-	GbExportDruidNetscapePrivate *p = e->priv;
-
-	g_free (p);
-	
-	G_OBJECT_CLASS (gb_export_druid_class)->finalize (o);
-
+	G_OBJECT_CLASS (gb_export_druid_netscape_parent_class)->finalize (o);
 }
 
 GbExportDruidNetscape *
