@@ -22,26 +22,16 @@
 
 #include <glib/gi18n.h>
 #include "bookmarks-import-druid-mozilla.h"
-#include "gul-gobject-misc.h"
 #include "gul-string.h"
 #include "gul-general.h"
 
 #include "bookmarks-mozilla.h"
 
-/**
- * Private data
- */
-struct _GbImportDruidMozillaPrivate {
-
-};
-
 static GHashTable *strings = NULL;
 
 /**
  * Private functions, only availble from this file
  */
-static void		gb_import_druid_mozilla_class_init		(GbImportDruidMozillaClass *klass);
-static void		gb_import_druid_mozilla_init			(GbImportDruidMozilla *e);
 static void		gb_import_druid_mozilla_finalize_impl		(GObject *o);
 static void		gb_import_druid_mozilla_init_strings 		(void);
 static const char *	gb_import_druid_mozilla_get_string_impl		(GbImportDruid *d, 
@@ -49,15 +39,12 @@ static const char *	gb_import_druid_mozilla_get_string_impl		(GbImportDruid *d,
 static GSList *		gb_import_druid_mozilla_get_locations_impl	(GbImportDruid *d);
 static GbIO *		gb_import_druid_mozilla_get_io_impl		(GbImportDruid *d);
 
-static gpointer gb_import_druid_class;
 
 /**
  * ImportDruidMozilla object
  */
 
-MAKE_GET_TYPE (gb_import_druid_mozilla, "GbImportDruidMozilla", GbImportDruidMozilla, 
-	       gb_import_druid_mozilla_class_init, 
-	       gb_import_druid_mozilla_init, GB_TYPE_IMPORT_DRUID);
+G_DEFINE_TYPE (GbImportDruidMozilla, gb_import_druid_mozilla, GB_TYPE_IMPORT_DRUID);
 
 static void
 gb_import_druid_mozilla_class_init (GbImportDruidMozillaClass *klass)
@@ -66,27 +53,17 @@ gb_import_druid_mozilla_class_init (GbImportDruidMozillaClass *klass)
 	GB_IMPORT_DRUID_CLASS (klass)->get_io = gb_import_druid_mozilla_get_io_impl;
 	GB_IMPORT_DRUID_CLASS (klass)->get_string = gb_import_druid_mozilla_get_string_impl;
 	G_OBJECT_CLASS (klass)->finalize = gb_import_druid_mozilla_finalize_impl;
-	gb_import_druid_class = g_type_class_peek_parent (klass);
 }
 
 static void 
 gb_import_druid_mozilla_init (GbImportDruidMozilla *e)
 {
-	GbImportDruidMozillaPrivate *p = g_new0 (GbImportDruidMozillaPrivate, 1);
-	e->priv = p;
-	
 }
 
 static void
 gb_import_druid_mozilla_finalize_impl (GObject *o)
 {
-	GbImportDruidMozilla *e = GB_IMPORT_DRUID_MOZILLA (o);
-	GbImportDruidMozillaPrivate *p = e->priv;
-
-	g_free (p);
-	
-	G_OBJECT_CLASS (gb_import_druid_class)->finalize (o);
-
+	G_OBJECT_CLASS (gb_import_druid_mozilla_parent_class)->finalize (o);
 }
 
 GbImportDruidMozilla *
