@@ -21,11 +21,14 @@
 #endif
 
 #include "bookmarks-iterator.h"
-#include "gul-gobject-misc.h"
 
 /**
  * Private data
  */
+#define GB_ITERATOR_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GB_TYPE_ITERATOR, GbIteratorPrivate))
+
+
 struct _GbIteratorPrivate {
 	gboolean recursive;
 	GbIterator *current_item_iterator;
@@ -35,31 +38,28 @@ struct _GbIteratorPrivate {
 /**
  * Private functions, only availble from this file
  */
-static void		gb_iterator_class_init		(GbIteratorClass *klass);
-static void		gb_iterator_init		(GbIterator *i);
 static void		gb_iterator_finalize_impl	(GObject *o);
 static void		gb_iterator_set_current_folder	(GbIterator *i, GbFolder *f);
 
-static gpointer g_object_class;
 
 /**
  * Iterator object
  */
 
-MAKE_GET_TYPE (gb_iterator, "GbIterator", GbIterator, gb_iterator_class_init, 
-	       gb_iterator_init, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GbIterator, gb_iterator, G_TYPE_OBJECT);
 
 static void
 gb_iterator_class_init (GbIteratorClass *klass)
 {
 	G_OBJECT_CLASS (klass)->finalize = gb_iterator_finalize_impl;
-	g_object_class = g_type_class_peek_parent (klass);
+
+	g_type_class_add_private (klass, sizeof (GbIteratorPrivate));
 }
 
 static void 
 gb_iterator_init (GbIterator *e)
 {
-	GbIteratorPrivate *p = g_new0 (GbIteratorPrivate, 1);
+	GbIteratorPrivate *p = GB_ITERATOR_GET_PRIVATE (e);
 	e->priv = p;
 }
 
@@ -69,9 +69,8 @@ gb_iterator_finalize_impl (GObject *o)
 	GbIterator *e = GB_ITERATOR (o);
 
 	gb_iterator_set_current_folder (e, NULL);
-	g_free (e->priv);
 
-	G_OBJECT_CLASS (g_object_class)->finalize (o);
+	G_OBJECT_CLASS (gb_iterator_parent_class)->finalize (o);
 }
 
 GbIterator *
@@ -79,7 +78,10 @@ gb_iterator_new (GbBookmarkSet *set)
 {
 	GbIterator *ret = g_object_new (GB_TYPE_ITERATOR, NULL);
 	ret->priv->next_item = GB_BOOKMARK (set->root);
-	g_object_ref (G_OBJECT (ret->priv->next_item));
+	if (ret->priv->next_item)
+	{
+		g_object_ref (G_OBJECT (ret->priv->next_item));
+	}
 	ret->priv->recursive = TRUE;
 	return ret;
 }
