@@ -30,7 +30,6 @@
 
 #include "bookmarks-netscape.h"
 #include "bookmarks-netscape-mozilla.h"
-#include "gul-gobject-misc.h"
 #include "gul-general.h"
 
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
@@ -54,8 +53,6 @@ typedef enum
 /**
  * Private functions, only availble from this file
  */
-static void		gb_io_netscape_class_init		(GbIONetscapeClass *klass);
-static void		gb_io_netscape_init			(GbIONetscape *io);
 static GbBookmarkSet *	gb_io_netscape_load_from_file		(GbIO *io, const gchar *filename);
 static GbBookmarkSet *	gb_io_netscape_load_from_string		(GbIO *io, const gchar *data);
 static gboolean		gb_io_netscape_save_to_file		(GbIO *io, GbBookmarkSet *set,
@@ -69,8 +66,7 @@ static GList *		gb_io_netscape_extensions 		(GbIO *io);
  * GbIONetscape object
  */
 
-MAKE_GET_TYPE (gb_io_netscape, "GbIONetscape", GbIONetscape, gb_io_netscape_class_init, 
-	       gb_io_netscape_init, GB_TYPE_IO);
+G_DEFINE_TYPE (GbIONetscape, gb_io_netscape, GB_TYPE_IO);
 
 static void
 gb_io_netscape_class_init (GbIONetscapeClass *klass)
