@@ -1,3 +1,4 @@
+/* -*- mode: c; c-style: k&r; c-basic-offset: 8 -*- */
 /*
  *  Copyright (C) 2002  Ricardo Fernndez Pascual
  *
@@ -23,11 +24,11 @@
 #include "bookmarks-smart-site-tb-widget.h"
 #include "galeon-marshal.h"
 #include "pixbuf-cache.h"
-#include "gul-gobject-misc.h"
 #include "gul-gui.h"
 #include "bookmarks-widgets-private.h"
 #include "bookmarks-single-editor.h"
 #include "bookmarks-dnd.h"
+#include "bookmarks-context-menu.h"
 #include "galeon-dnd.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
@@ -36,18 +37,22 @@
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkimage.h>
-#include <gtk/gtkcombo.h>
+#include <gtk/gtkcomboboxentry.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkeditable.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 /**
  * Private data
  */
+#define GB_SMART_SITE_TB_WIDGET_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GB_TYPE_SMART_SITE_TB_WIDGET, GbSmartSiteTbWidgetPrivate))
+
+
 struct _GbSmartSiteTbWidgetPrivate {
 	GtkWidget *mainwid;
 	GtkWidget *main_button;
@@ -78,8 +83,6 @@ const GtkTargetEntry text_drop_types[] =
 /**
  * Private functions, only availble from this file
  */
-static void		gb_smart_site_tb_widget_class_init	(GbSmartSiteTbWidgetClass *klass);
-static void		gb_smart_site_tb_widget_init		(GbSmartSiteTbWidget *w);
 static void		gb_smart_site_tb_widget_finalize_impl	(GObject *o);
 static void		gb_smart_site_tb_widget_rebuild_impl	(GbTbWidget *w);
 static void		gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w);
@@ -109,35 +112,37 @@ static void		gb_smart_site_tb_widget_drag_data_received_cb (GtkWidget *widget,
 								       guint info,
 								       guint t,
 								       GbSmartSiteTbWidget *w);
+static gboolean 	gb_smart_site_tb_widget_entries_empty	(GbSmartSiteTbWidget *w);
+static void		gb_smart_site_tb_widget_do_the_smart_thing (GbSmartSiteTbWidget *w, 
+								    GbBookmarkActivatedFlags flags);
+
 
 
-static gpointer gb_tb_widget_class;
 
 /**
  * GbSmartSiteTbWidget object
  */
 
-MAKE_GET_TYPE (gb_smart_site_tb_widget, "GbSmartSiteTbWidget", GbSmartSiteTbWidget, 
-	       gb_smart_site_tb_widget_class_init, 
-	       gb_smart_site_tb_widget_init, GB_TYPE_TB_WIDGET);
+G_DEFINE_TYPE (GbSmartSiteTbWidget, gb_smart_site_tb_widget, GB_TYPE_TB_WIDGET);
 
 static void
 gb_smart_site_tb_widget_class_init (GbSmartSiteTbWidgetClass *klass)
 {
 	G_OBJECT_CLASS (klass)->finalize = gb_smart_site_tb_widget_finalize_impl;
 	GB_TB_WIDGET_CLASS (klass)->rebuild = gb_smart_site_tb_widget_rebuild_impl;
-	gb_tb_widget_class = g_type_class_peek_parent (klass);
 
 	eel_gconf_notification_add 
 		(CONF_BOOKMARKS_HISTORY, gb_smart_site_tb_widget_conf_changed_cb, NULL);
 	eel_gconf_notification_add 
 		(CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS, gb_smart_site_tb_widget_conf_changed_cb, NULL);
+
+	g_type_class_add_private (klass, sizeof (GbSmartSiteTbWidgetPrivate));
 }
 
 static void 
 gb_smart_site_tb_widget_init (GbSmartSiteTbWidget *w)
 {
-	GbSmartSiteTbWidgetPrivate *p = g_new0 (GbSmartSiteTbWidgetPrivate, 1);
+	GbSmartSiteTbWidgetPrivate *p = GB_SMART_SITE_TB_WIDGET_GET_PRIVATE (w);
 	w->priv = p;
 
 	all_instances = g_slist_prepend (all_instances, w);
@@ -155,9 +160,8 @@ gb_smart_site_tb_widget_finalize_impl (GObject *o)
 	g_free (p->combos);
 	g_free (p->entries_sizes);
 
-	g_free (p);
 	
-	G_OBJECT_CLASS (gb_tb_widget_class)->finalize (o);
+	G_OBJECT_CLASS (gb_smart_site_tb_widget_parent_class)->finalize (o);
 }
 
 GbSmartSiteTbWidget *
@@ -189,6 +193,7 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 	GbSmartSiteTbWidgetPrivate *p = w->priv;
 	GbSmartSite *ss = GB_SMART_SITE (gb_tb_widget_get_bookmark (GB_TB_WIDGET (w)));
 	int num_entries = gb_smart_site_get_num_fields (ss);
+	gboolean hide_arrows = eel_gconf_get_boolean (CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS);
 	int i;
 
 	p->num_entries = num_entries;
@@ -237,11 +242,18 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 		if (eel_gconf_get_boolean (CONF_BOOKMARKS_HISTORY))
 		{
 			GSList *history = gb_smart_site_get_history (ss, i);
-			GList *history_l = gslist_to_glist (history);
+			GList *l, *history_l = gslist_to_glist (history);
+
 			history_l = g_list_prepend (history_l, "");
-			p->combos[i] = gtk_combo_new ();
-			p->entries[i] = GTK_COMBO (p->combos[i])->entry;
-			gtk_combo_set_popdown_strings (GTK_COMBO (p->combos[i]), history_l);
+			p->combos[i] = gtk_combo_box_entry_new_text();
+			p->entries[i] = GTK_BIN(p->combos[i])->child;
+
+			for (l = history_l; l; l = l->next)
+			{
+				gtk_combo_box_append_text(GTK_COMBO_BOX(p->combos[i]),
+				                          (const gchar *)l->data);
+			}
+
 			g_slist_foreach (history, (GFunc) g_free, NULL);
 			g_slist_free (history);
 			g_list_free (history_l);
@@ -253,7 +265,7 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 		}
 
 		p->entries_sizes[i] = gb_smart_site_get_entry_size (ss, i);
-		if (!ss->folded)
+		if (!ss->folded || hide_arrows)
 		{
 			gtk_widget_show (p->combos[i] ? p->combos[i] : p->entries[i]);
 		}
@@ -266,7 +278,7 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 	p->entries[num_entries] = NULL;
 	p->combos[num_entries] = NULL;
 
-	if (!eel_gconf_get_boolean (CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS))
+	if (!hide_arrows)
 	{
 
 		if (ss->folded)
@@ -290,6 +302,75 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 			  G_CALLBACK (gb_smart_site_tb_widget_entry_width_changed_cb), w);
 }
 
+static void
+gb_smart_site_context_menu_bookmark_activated_cb (GbContextMenu *gm, 
+						  GbBookmarkEventActivated *ev,
+						  GbSmartSiteTbWidget *w)
+{
+	if (gb_smart_site_tb_widget_entries_empty (w))
+	{
+		gb_smart_site_tb_widget_do_the_smart_thing (w, ev->flags);
+	}
+	else
+	{
+		g_signal_emit_by_name (w, "bookmark-activated", ev);
+	}
+}
+
+static void
+gb_smart_site_tb_widget_popup_menu (GbSmartSiteTbWidget *w, GdkEventButton *event)
+{
+	GbBookmark *b = gb_tb_widget_get_bookmark (GB_TB_WIDGET (w));
+	GbLocationSource *location_source = gb_tb_widget_get_location_source (GB_TB_WIDGET (w));
+	GbContextMenu *cm = gb_context_menu_new ();
+	gb_context_menu_set_bookmark (cm, b);
+	gb_context_menu_set_location_source (cm, location_source);
+	
+	g_signal_connect (cm, "bookmark-activated", 
+			  G_CALLBACK (gb_smart_site_context_menu_bookmark_activated_cb), 
+			  w);
+	gb_context_menu_popup (cm, event);
+	
+	g_object_unref (cm);
+}
+
+static gboolean
+gb_smart_site_tb_widget_popup_menu_cb (GtkWidget *b, GbSmartSiteTbWidget *w)
+{
+	gb_smart_site_tb_widget_popup_menu (w, NULL);
+	return TRUE;
+}
+
+static gboolean
+gb_smart_site_tb_widget_button_press_cb (GtkWidget *wid, 
+					 GdkEventButton *event, 
+					 GbSmartSiteTbWidget *w)
+{
+	GbBookmark *b = gb_tb_widget_get_bookmark (GB_TB_WIDGET (w));
+	g_return_val_if_fail (GB_IS_SMART_SITE (b), FALSE);
+	
+	if (b && event->button == 2)
+	{
+		if (gb_smart_site_tb_widget_entries_empty (w))
+		{
+			gb_smart_site_tb_widget_do_the_smart_thing (w, GB_BAF_NEW_TAB_OR_WINDOW);
+		}
+		else
+		{
+			gb_activated_event (w, b, GB_SITE (b)->url, 
+					    GB_BAF_NEW_TAB_OR_WINDOW, (GdkEvent *) event);
+		}
+		return TRUE;
+	}
+	else if (event->button == 3)
+	{
+		gb_smart_site_tb_widget_popup_menu (w, event);
+		return TRUE;
+	}
+	return FALSE;
+
+}
+
 static void
 gb_smart_site_tb_widget_rebuild_impl (GbTbWidget *w)
 {
@@ -321,8 +402,12 @@ gb_smart_site_tb_widget_rebuild_impl (GbTbWidget *w)
 	gtk_widget_show (GTK_WIDGET (p->main_button));
 	gb_widgets_fill_tb_item (GTK_CONTAINER (p->main_button), GB_BOOKMARK (site), GB_SITE (site)->url);
 
-	gb_tb_widget_setup_context_menu (w, p->main_button);
-	gb_bookmark_dnd_drag_source_set (p->main_button, GB_BOOKMARK (site));
+	g_signal_connect (p->main_button, "popup_menu", 
+			  G_CALLBACK (gb_smart_site_tb_widget_popup_menu_cb), w);
+	g_signal_connect (p->main_button, "button_press_event",
+			  G_CALLBACK (gb_smart_site_tb_widget_button_press_cb), w);
+
+	gb_bookmark_dnd_drag_source_set (GTK_BIN (p->main_button)->child, GB_BOOKMARK (site));
 
 	if (gb_smart_site_get_num_fields (site) == 1 )
 	{
@@ -352,6 +437,8 @@ gb_smart_site_tb_widget_unfold_cb (GtkWidget *button, GbSmartSiteTbWidget *w)
 {
 	GbSmartSite *ss = GB_SMART_SITE (gb_tb_widget_get_bookmark (GB_TB_WIDGET (w)));
 	gb_smart_site_set_folded (ss, FALSE);
+	/* bug 89338 */
+	gtk_widget_grab_focus (w->priv->entries[0]);
 }
 
 static void
@@ -359,13 +446,14 @@ gb_smart_site_tb_widget_visibility_changed_cb (GbSmartSite *ss, GbSmartSiteTbWid
 {
 	GbSmartSiteTbWidgetPrivate *p = w->priv;
 	gint i;
+	gboolean hide_arrows = eel_gconf_get_boolean (CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS);
 
 	g_return_if_fail (gb_tb_widget_get_bookmark (GB_TB_WIDGET (w)) == GB_BOOKMARK (ss));
 	g_return_if_fail (p->entries && p->combos);
 
 	for (i = 0; p->entries[i]; i++)
 	{
-		if (!ss->folded)
+		if (!ss->folded || hide_arrows)
 		{
 			gtk_widget_show (p->combos[i] ? p->combos[i] : p->entries[i]);
 		}
@@ -375,7 +463,7 @@ gb_smart_site_tb_widget_visibility_changed_cb (GbSmartSite *ss, GbSmartSiteTbWid
 		}
 	}
 	
-	if (!eel_gconf_get_boolean (CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS) )
+	if (!hide_arrows)
 	{
 		if (ss->folded)
 		{
@@ -397,7 +485,7 @@ gb_smart_site_tb_widget_visibility_changed_cb (GbSmartSite *ss, GbSmartSiteTbWid
 
 static void
 gb_smart_site_tb_widget_do_the_smart_thing (GbSmartSiteTbWidget *w, 
-					     GbBookmarkActivatedFlags flags)
+					    GbBookmarkActivatedFlags flags)
 {
 	GbSmartSiteTbWidgetPrivate *p = w->priv;
 	gchar *url;
@@ -504,29 +592,6 @@ gb_smart_site_tb_widget_entry_width_changed_cb (GbSmartSite *b, gint index, guin
 	gtk_widget_set_size_request (p->entries[index], p->entries_sizes[index], -1);
 }
 
-
-static gboolean
-gb_smart_site_tb_widget_button_press_cb (GtkWidget *wid, 
-					 GdkEventButton *event, 
-					 GbSmartSiteTbWidget *w)
-{
-	GbBookmark *b = gb_tb_widget_get_bookmark (GB_TB_WIDGET (w));
-	if (b && event->button == 2)
-	{
-		if (gb_smart_site_tb_widget_entries_empty (w))
-		{
-			gb_smart_site_tb_widget_do_the_smart_thing (w, GB_BAF_NEW_TAB_OR_WINDOW);
-		}
-		else
-		{
-			gb_activated_event (w, b, GB_SITE (b)->url, 
-					    GB_BAF_NEW_TAB_OR_WINDOW, (GdkEvent *) event);
-		}
-		return TRUE;
-	}
-	return FALSE;
-}
-
 static void
 gb_smart_site_tb_widget_history_changed_cb (GbSmartSite *ss, GbSmartSiteTbWidget *w)
 {
@@ -538,11 +603,22 @@ gb_smart_site_tb_widget_history_changed_cb (GbSmartSite *ss, GbSmartSiteTbWidget
 	for (i = 0; i < p->num_entries; ++i)
 		if (p->combos[i])
 		{
+			GList *l;
+
 			history = gb_smart_site_get_history (ss, i);
 			history_l = gslist_to_glist (history);
 			history_l = g_list_prepend (history_l, "");
 
-			gtk_combo_set_popdown_strings (GTK_COMBO (p->combos[i]), history_l);
+			gtk_combo_box_set_active(GTK_COMBO_BOX(p->combos[i]), -1);
+			gtk_list_store_clear(GTK_LIST_STORE(
+			    gtk_combo_box_get_model(GTK_COMBO_BOX(p->combos[i]))));
+
+			for (l = history_l; l; l = l->next)
+			{
+				gtk_combo_box_append_text(GTK_COMBO_BOX(p->combos[i]),
+				                          (const gchar *)l->data);
+			}
+
 			g_slist_foreach (history, (GFunc) g_free, NULL);
 			g_slist_free (history);
 			g_list_free (history_l);
@@ -583,11 +659,11 @@ gb_smart_site_tb_widget_drag_data_received_cb (GtkWidget *widget,
 
 	if (info == GALEON_DND_TEXT)
 	{
-		str = g_strdup (selection_data->data);
+		str = g_strdup ((char*)selection_data->data);
 	} 
 	else
 	{
-		str = gtk_selection_data_get_text (selection_data);
+		str = (char*)gtk_selection_data_get_text (selection_data);
 	}
 
 	if (str == NULL || str[0] == '\0') return;
