@@ -1,3 +1,4 @@
+/* -*- mode: c c-style: k&r c-basic-offset: 8 -*- */
 /*
  *  Copyright (C) 2002  Ricardo Fernndez Pascual
  *
@@ -23,26 +24,26 @@
 #include "bookmarks-tb-widget.h"
 #include "bookmarks-context-menu.h"
 #include "galeon-marshal.h"
-#include "gul-gobject-misc.h"
+#include "galeon-debug.h"
 
-#include <libgnome/gnome-i18n.h>
-
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
+#include <glib/gi18n.h>
 
 /**
  * Private data
  */
+#define GB_TB_WIDGET_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GB_TYPE_TB_WIDGET, GbTbWidgetPrivate))
+
+
 struct _GbTbWidgetPrivate {
 	GbBookmark *bm;
 	GbLocationSource *location_source;
+	GtkStatusbar *statusbar;
 };
 
 /**
  * Private functions, only availble from this file
  */
-static void		gb_tb_widget_class_init		(GbTbWidgetClass *klass);
-static void		gb_tb_widget_init		(GbTbWidget *w);
 static void		gb_tb_widget_finalize_impl 	(GObject *o);
 
 static void		gb_tb_widget_modified_cb	(GbBookmark *b, GbTbWidget *w);
@@ -59,7 +60,6 @@ static void		gb_tb_widget_get_property_impl	(GObject *object,
 							 GValue *value,
 							 GParamSpec *pspec);
 
-static gpointer gtk_hbox_class;
 
 /* signals enums and ids */
 enum GbTbWidgetSignalsEnum {
@@ -78,8 +78,7 @@ enum {
  * GbTbWidget object
  */
 
-MAKE_GET_TYPE (gb_tb_widget, "GbTbWidget", GbTbWidget, gb_tb_widget_class_init, 
-	       gb_tb_widget_init, GTK_TYPE_HBOX);
+G_DEFINE_TYPE (GbTbWidget, gb_tb_widget, GTK_TYPE_HBOX);
 
 static void
 gb_tb_widget_class_init (GbTbWidgetClass *klass)
@@ -88,7 +87,6 @@ gb_tb_widget_class_init (GbTbWidgetClass *klass)
 	G_OBJECT_CLASS (klass)->set_property = gb_tb_widget_set_property_impl;
 	G_OBJECT_CLASS (klass)->get_property = gb_tb_widget_get_property_impl;
 	klass->rebuild = gb_tb_widget_rebuild_impl;
-	gtk_hbox_class = g_type_class_peek_parent (klass);
 
 	GbTbWidgetSignals[GB_TB_WIDGET_BOOKMARK_ACTIVATED] = g_signal_new (
 		"bookmark-activated", G_OBJECT_CLASS_TYPE (klass),  
@@ -106,12 +104,14 @@ gb_tb_widget_class_init (GbTbWidgetClass *klass)
 							      GB_TYPE_BOOKMARK,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+
+	g_type_class_add_private (klass, sizeof (GbTbWidgetPrivate));
 }
 
 static void 
 gb_tb_widget_init (GbTbWidget *w)
 {
-	GbTbWidgetPrivate *p = g_new0 (GbTbWidgetPrivate, 1);
+	GbTbWidgetPrivate *p = GB_TB_WIDGET_GET_PRIVATE (w);
 	w->priv = p;
 }
 
@@ -128,14 +128,16 @@ gb_tb_widget_finalize_impl (GObject *o)
 		g_signal_handlers_disconnect_matched (p->bm, G_SIGNAL_MATCH_DATA, 0, 0, 
 						      NULL, NULL, w);
 		g_object_unref (G_OBJECT (p->bm));
-
 	}
 
-	g_free (p);
+	if (p->statusbar)
+	{
+		g_object_unref (p->statusbar);
+	}
 
-	DEBUG_MSG (("GbTbWidget finalized\n"));
+	LOG ("GbTbWidget finalized");
 
-	G_OBJECT_CLASS (gtk_hbox_class)->finalize (o);
+	G_OBJECT_CLASS (gb_tb_widget_parent_class)->finalize (o);
 }
 
 void
@@ -273,3 +275,26 @@ gb_tb_widget_get_location_source (GbTbWidget *w)
 	return p->location_source;
 }
 
+void
+gb_tb_widget_set_statusbar (GbTbWidget *w, GtkStatusbar *statusbar)
+{
+	GbTbWidgetPrivate *p = w->priv;
+	g_return_if_fail (!statusbar || GTK_IS_STATUSBAR (statusbar));
+	if (p->statusbar)
+	{
+		g_object_unref (p->statusbar);
+		p->statusbar = NULL;
+	}
+
+	if (statusbar)
+	{
+		p->statusbar = g_object_ref (statusbar);
+	}	
+}
+
+GtkStatusbar *
+gb_tb_widget_get_statusbar (GbTbWidget *w)
+{
+     return w->priv->statusbar;
+}
+
