@@ -1,3 +1,4 @@
+/* -*- mode: c; c-style: k&r; c-basic-offset: 8 -*- */
 /*
  *  Copyright (C) 2002  Ricardo Fernndez Pascual
  *
@@ -24,7 +25,7 @@
 #include <gtk/gtkwidget.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libxml/tree.h>
-
+#include "galeon-auto-bookmarks-source.h"
 
 G_BEGIN_DECLS
 
@@ -40,6 +41,8 @@ typedef struct _GbFolder GbFolder;
 typedef struct _GbFolderClass GbFolderClass;
 typedef struct _GbSeparator GbSeparator;
 typedef struct _GbSeparatorClass GbSeparatorClass;
+typedef struct _GbVFolder GbVFolder;
+typedef struct _GbVFolderClass GbVFolderClass;
 typedef struct _GbAutoFolder GbAutoFolder;
 typedef struct _GbAutoFolderClass GbAutoFolderClass;
 typedef struct _GbAliasPlaceholder GbAliasPlaceholder;
@@ -51,6 +54,13 @@ typedef struct _GbIOClass GbIOClass;
 typedef struct _GbTbWidget GbTbWidget;
 typedef struct _GbIconProvider GbIconProvider;
 
+typedef enum
+{
+	GB_MATCH_TYPE_AND,
+	GB_MATCH_TYPE_OR,
+	GB_MATCH_TYPE_EXACT
+} GbMatchType;
+
 /**
  * General functions that affect the bookmark system.
  */
@@ -71,19 +81,21 @@ GbIconProvider *	gb_system_get_icon_provider	(void);
 #define GB_BOOKMARK_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), GB_TYPE_BOOKMARK,\
 					 GbBookmarkClass))
 
+typedef struct _GbBookmarkCopyContext GbBookmarkCopyContext;
+
 struct _GbBookmarkClass 
 {
 	GObjectClass parent_class;
 
 	/* virtual methods */
 	void		(*gb_bookmark_set_set)		(GbBookmark *b, GbBookmarkSet *set);
-	GbBookmark *	(*gb_bookmark_copy)		(GbBookmark *b);
+	GbBookmark *	(*gb_bookmark_copy)		(GbBookmark *b, GbBookmarkCopyContext *cc);
 	GbBookmark *	(*gb_bookmark_alias_create)	(GbBookmark *b, GbAliasPlaceholder *ap);
 	GbTbWidget *	(*gb_bookmark_create_toolbar_widget) (GbBookmark *b);
 
 	/* signals */
 	void		(*gb_bookmark_modified)		(GbBookmark *b);
-	void		(*gb_bookmark_name_modified)	(GbBookmark *b, const gchar *new_name);
+	void		(*gb_bookmark_replaced)		(GbBookmark *b, GbBookmark *replacement);
 };
 
 /* Remember: fields are public read-only */
@@ -141,6 +153,7 @@ struct _GbBookmark
 
 GType		gb_bookmark_get_type			(void);
 void		gb_bookmark_set_id			(GbBookmark *b, const gchar *val);
+const char *	gb_bookmark_get_id			(GbBookmark *b);
 void		gb_bookmark_set_xbel_node		(GbBookmark *b, xmlNodePtr node);
 void		gb_bookmark_set_name			(GbBookmark *b, const gchar *val);
 void		gb_bookmark_set_nick			(GbBookmark *b, const gchar *val);
@@ -166,6 +179,7 @@ void		gb_bookmark_emit_changed		(GbBookmark *b);
 GbBookmark *	gb_bookmark_add_alias_under		(GbBookmark *b, GbFolder *newparent);
 GbBookmark *	gb_bookmark_ensure_alias_under		(GbBookmark *b, GbFolder *newparent);
 GSList *	gb_bookmark_get_all_alias_parents	(GbBookmark *b);
+void		gb_bookmark_replace			(GbBookmark *b, GbBookmark *replacement);
 
 /**
  * Site object
@@ -209,6 +223,7 @@ void 		gb_site_set_url			(GbSite *b, const char *val);
 void		gb_site_set_time_visited 	(GbSite *b, GTime val);
 void		gb_site_set_time_visited_now	(GbSite *b);
 void		gb_site_set_accel		(GbSite *b, guint accel_key, guint accel_mods);
+GbSmartSite *	gb_site_make_smart		(GbSite *b, const gchar *smarturl);
 
 /**
  * SmartSite object
@@ -262,6 +277,7 @@ void		gb_smart_site_set_encoding	(GbSmartSite *b, const gchar *encoding);
 GSList *	gb_smart_site_get_history	(GbSmartSite *b, int entry_index);
 void		gb_smart_site_set_history	(GbSmartSite *b, int entry_index, const GSList *history);
 void		gb_smart_site_prepend_history	(GbSmartSite *b, int entry_index, const gchar *item);
+GbSite *	gb_smart_site_make_dumb		(GbSmartSite *b);
 
 /**
  * Folder object
@@ -281,6 +297,9 @@ struct _GbFolderClass
 {
 	GbBookmarkClass parent_class;
 
+	/* virtual methods */
+	gboolean	(*gb_folder_is_autogenerated)	(GbFolder *f);
+
 	/* signals */
 	void		(*gb_folder_child_modified)	(GbFolder *f, GbBookmark *c);
 	void		(*gb_folder_child_added)	(GbFolder *f, GbBookmark *c, gint position);
@@ -321,10 +340,12 @@ gint		gb_folder_get_child_index		(GbFolder *p, GbBookmark *b);
 gboolean	gb_folder_is_ancestor			(GbFolder *p, GbBookmark *b);
 gboolean 	gb_folder_is_default_folder		(GbFolder *f);
 GSList * 	gb_folder_list_children			(GbFolder *f);
+GSList * 	gb_folder_list_children_reversed	(GbFolder *f);
 void		gb_folder_sort				(GbFolder *f, gboolean folders_first, 
 							 gboolean recursive);
 gboolean 	gb_folder_has_child_or_alias		(GbFolder *f, GbBookmark *c);
 void	 	gb_folder_remove_child_or_aliases	(GbFolder *f, GbBookmark *c);
+gboolean	gb_folder_is_autogenerated 		(GbFolder *f);
 
 /**
  * Separator object
@@ -353,6 +374,76 @@ struct _GbSeparator
 GType		gb_separator_get_type	(void);
 GbSeparator *	gb_separator_new 	(GbBookmarkSet *set);
 
+/**
+ * VFolder object
+ */
+
+#define GB_TYPE_V_FOLDER 		(gb_v_folder_get_type())
+#define GB_V_FOLDER(object) 		(G_TYPE_CHECK_INSTANCE_CAST((object), GB_TYPE_V_FOLDER,\
+					 GbVFolder))
+#define GB_V_FOLDER_CLASS(klass) 	(G_TYPE_CHECK_CLASS_CAST((klass), GB_TYPE_V_FOLDER,\
+					 GbVFolderClass))
+#define GB_IS_V_FOLDER(object) 		(G_TYPE_CHECK_INSTANCE_TYPE((object), \
+					 GB_TYPE_V_FOLDER))
+#define GB_IS_V_FOLDER_CLASS(klass) 	(G_TYPE_CHECK_CLASS_TYPE((klass), GB_TYPE_V_FOLDER))
+#define GB_V_FOLDER_GET_CLASS(obj) 	(G_TYPE_INSTANCE_GET_CLASS((obj), GB_TYPE_V_FOLDER,\
+					 GbVFolderClass))
+
+struct _GbVFolderClass 
+{
+	GbFolderClass parent_class;
+};
+
+struct _GbVFolder
+{
+	GbFolder parent_object;
+     
+	gchar *search_text;
+	gchar *search_casefold_text;
+	gchar **search_text_split;
+	gchar **search_casefold_text_split;
+	gboolean search_match_case;
+	gboolean search_include_folders;
+	gboolean search_include_sites;
+	gboolean search_look_in_name;
+	gboolean search_look_in_location;
+	gboolean search_look_in_notes;
+	GbMatchType search_match_type;
+	gboolean search_recently_visited;
+	GTime search_recently_visited_time;
+	gboolean search_recently_created;
+	GTime search_recently_created_time;
+
+	gboolean dirty;
+};
+
+GType		gb_v_folder_get_type		(void);
+GbVFolder *	gb_v_folder_new			(GbBookmarkSet *set, const gchar *name);
+void		gb_v_folder_refresh		(GbVFolder *af);
+void		gb_v_folder_set_search_text	(GbVFolder *af, const gchar *text);
+void		gb_v_folder_set_search_options	(GbVFolder *af, const gchar *options);
+void		gb_v_folder_set_match_case	(GbVFolder *af, gboolean v);
+void		gb_v_folder_set_include_folders	(GbVFolder *af, gboolean v);
+void		gb_v_folder_set_include_sites	(GbVFolder *af, gboolean v);
+void		gb_v_folder_set_look_in_name	(GbVFolder *af, gboolean v);
+void		gb_v_folder_set_look_in_location (GbVFolder *af, gboolean v);
+void		gb_v_folder_set_look_in_notes	(GbVFolder *af, gboolean v);
+void		gb_v_folder_set_match_type	(GbVFolder *af, GbMatchType v);
+void		gb_v_folder_set_rencently_visited (GbVFolder *af, gboolean v, gint days);
+void		gb_v_folder_set_rencently_created (GbVFolder *af, gboolean v, gint days);
+const gchar *	gb_v_folder_get_search_text	(GbVFolder *af);
+gchar *		gb_v_folder_get_search_options	(GbVFolder *af);
+gboolean	gb_v_folder_get_match_case	(GbVFolder *af);
+gboolean	gb_v_folder_get_include_folders	(GbVFolder *af);
+gboolean	gb_v_folder_get_include_sites	(GbVFolder *af);
+gboolean	gb_v_folder_get_look_in_name	(GbVFolder *af);
+gboolean	gb_v_folder_get_look_in_location (GbVFolder *af);
+gboolean	gb_v_folder_get_look_in_notes	(GbVFolder *af);
+GbMatchType	 gb_v_folder_get_match_type	(GbVFolder *af);
+gboolean	gb_v_folder_get_rencently_visited (GbVFolder *af, gint *days);
+gboolean	gb_v_folder_get_rencently_created (GbVFolder *af, gint *days);
+void		gb_v_folder_set_dirty		(GbVFolder *af);
+
 /**
  * AutoFolder object
  */
@@ -376,11 +467,42 @@ struct _GbAutoFolderClass
 struct _GbAutoFolder
 {
 	GbFolder parent_object;
+
+	gchar *search_text;
+	gchar *search_casefold_text;
+	gchar **search_text_split;
+	gchar **search_casefold_text_split;
+	gboolean search_match_case;
+	gboolean search_look_in_title;
+	gboolean search_look_in_url;
+	GbMatchType search_match_type;
+	gboolean group_by_host;
+	GaleonAutoBookmarksScoringMethod scoring;
+	gint max_matches;
 };
 
-GType		gb_auto_folder_get_type	(void);
-GbAutoFolder *	gb_auto_folder_new	(GbBookmarkSet *set, const gchar *name);
-void		gb_auto_folder_refresh	(void);
+GType		gb_auto_folder_get_type			(void);
+GbAutoFolder *	gb_auto_folder_new			(GbBookmarkSet *set, const gchar *name);
+void		gb_auto_folder_refresh			(GbAutoFolder *af);
+void		gb_auto_folder_set_search_text		(GbAutoFolder *af, const gchar *text);
+void		gb_auto_folder_set_options		(GbAutoFolder *af, const gchar *options);
+void		gb_auto_folder_set_match_case		(GbAutoFolder *af, gboolean v);
+void		gb_auto_folder_set_look_in_title	(GbAutoFolder *af, gboolean v);
+void		gb_auto_folder_set_look_in_url		(GbAutoFolder *af, gboolean v);
+void		gb_auto_folder_set_match_type		(GbAutoFolder *af, GbMatchType v);
+void		gb_auto_folder_set_group_by_host	(GbAutoFolder *af, gboolean v);
+void		gb_auto_folder_set_scoring_method	(GbAutoFolder *af, 
+							 GaleonAutoBookmarksScoringMethod scoring);
+
+const gchar *	gb_auto_folder_get_search_text		(GbAutoFolder *af);
+gchar *		gb_auto_folder_get_options		(GbAutoFolder *af);
+gboolean	gb_auto_folder_get_match_case		(GbAutoFolder *af);
+gboolean	gb_auto_folder_get_look_in_title	(GbAutoFolder *af);
+gboolean	gb_auto_folder_get_look_in_url		(GbAutoFolder *af);
+GbMatchType	gb_auto_folder_get_match_type		(GbAutoFolder *af);
+gboolean	gb_auto_folder_get_group_by_host	(GbAutoFolder *af);
+GaleonAutoBookmarksScoringMethod 
+		gb_auto_folder_get_scoring_method	(GbAutoFolder *af);
 
 /**
  * AliasPlaceholder object
@@ -475,15 +597,22 @@ struct _GbBookmarkSet
 	/* all bookmarks in preorder, or NULL */
 	GSList *all_bookmarks;
 	GSList *all_bookmarks_noalias;
+
 	GSList *context_bookmarks;
+	GSList *v_folders;
+	GSList *auto_folders;
 
 	GbIO *io;
 	gchar *filename;
 	GbFileFormatVersion file_format_version;
 	gboolean needs_saving;
 	guint autosave_timeout_id;
+	guint search_timeout_id;
+	guint autobookmarks_timeout_id;
 
 	guint max_smart_site_history_items;
+
+	GaleonAutoBookmarksSource *autobookmarks_source;
 };
 
 GType		gb_bookmark_set_get_type		(void);
@@ -498,6 +627,8 @@ void		gb_bookmark_set_set_default_folder	(GbBookmarkSet *set, GbFolder *default_
 void		gb_bookmark_set_resolve_aliases		(GbBookmarkSet *set);
 GSList *	gb_bookmark_set_get_toolbars	 	(GbBookmarkSet *set);
 const GSList *	gb_bookmark_set_get_context_bookmarks 	(GbBookmarkSet *set);
+const GSList *	gb_bookmark_set_get_v_folders	 	(GbBookmarkSet *set);
+const GSList *	gb_bookmark_set_get_auto_folders 	(GbBookmarkSet *set);
 void		gb_bookmark_set_set_needs_saving	(GbBookmarkSet *set, gboolean needs_saving);
 void		gb_bookmark_set_set_auto_save		(GbBookmarkSet *set, guint interval);
 void		gb_bookmark_set_add_default		(GbBookmarkSet *set, GbBookmark *b);
@@ -510,6 +641,8 @@ gchar *		gb_bookmark_set_get_url_by_nick_and_args (GbBookmarkSet *set,
 							  const gchar *nick_and_args);
 const GSList *	gb_bookmark_set_get_all			(GbBookmarkSet *set);
 const GSList *	gb_bookmark_set_get_all_noalias		(GbBookmarkSet *set);
+void		gb_bookmark_set_set_autobookmarks_source (GbBookmarkSet *set, GaleonAutoBookmarksSource *src);
+GbBookmark *    gb_bookmark_set_get_bookmark_for_id      (GbBookmarkSet *b, const char *id);
 
 
 G_END_DECLS
