@@ -20,31 +20,33 @@
 #define GALEON_EMBED_UTILS_H
 
 #include "galeon-embed-persist.h"
+#include "galeon-embed-event.h"
 
 #include <gtk/gtkwidget.h>
-#include <bonobo/bonobo-ui-component.h>
 
 G_BEGIN_DECLS
 
-typedef struct
-{
-        const char *encoding;
-        gpointer data;
-} EncodingMenuData;
+void galeon_embed_utils_save_event_property     (GaleonEmbed *embed,
+						 GaleonEmbedEvent *event,
+						 gboolean always_ask_dest,
+						 gboolean show_progress,
+						 const char *fc_title,
+						 const char *property);
 
-void galeon_embed_utils_save 			(GtkWidget *window,
-			   	 		const char *default_dir_pref,
-				 		gboolean ask_dest,
-				 		gboolean ask_content,
-				 		GaleonEmbedPersist *persist);
-
-void galeon_embed_utils_build_charsets_submenu  (BonoboUIComponent *ui_component,
-						 const char *path,
-						 BonoboUIVerbFn fn,
-						 gpointer data);
+void galeon_embed_utils_download_event_property (GaleonEmbed *embed,
+						 GaleonEmbedEvent *event,
+						 gboolean always_ask_dest,
+						 const char *fc_title,
+						 const char *property);
 
 void galeon_embed_utils_nohandler_dialog_run    (GtkWidget *parent);
 
+void galeon_embed_utils_free_page_properties	(EmbedPageProperties *props);
+
+GList *galeon_embed_utils_get_user_stylesheets	(void);
+
+gchar *galeon_embed_utils_get_user_sheet_path   (const gchar *sheetfile);
+
 G_END_DECLS
 
 #endif
