@@ -9,11 +9,12 @@
 #include "config.h"
 #endif
 
-#include "MozillaPrivate.h"
+#include "GaleonUtils.h"
 #include "gul-glade.h"
 #include "galeon-embed-shell.h"
 #include "mozilla-cookie.h"
-#include "nsString.h"
+#include "GulString.h"
+#include "AutoJSContextStack.h"
 
 #include <gtk/gtkdialog.h>
 #include <gtk/gtklabel.h>
@@ -26,11 +27,11 @@
 
 #include <libgnome/gnome-triggers.h>
 #include <glib/gi18n.h>
-#include <libgnome/gnome-config.h>
 
 #include "CookiePromptService.h"
 #include "hig-alert.h"
 #include "gul-state.h"
+#include "gul-gui.h"
 
 /* gnome-config keys for saving state */
 #define STATE_COOKIE_REMEMBER_DECISION "Cookie/remember_decision"
@@ -107,42 +108,22 @@ higgy_indent_widget (GtkWidget *widget)
 	return hbox;
 }
 
-
-static GtkWidget *
-create_button_with_image (const gchar *text, const gchar *stock_icon)
-{
-	GtkWidget *button, *align, *hbox, *image, *label;
-
-	button = gtk_button_new ();
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-
-	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-	gtk_container_add (GTK_CONTAINER (button), align);
-
-	hbox = gtk_hbox_new (FALSE, 2);
-	gtk_container_add (GTK_CONTAINER (align), hbox);
-
-	image = gtk_image_new_from_stock (stock_icon, GTK_ICON_SIZE_BUTTON);
-	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-
-	label = gtk_label_new_with_mnemonic (text);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-
-	gtk_widget_show_all (button);
-	return button;
-}
-
-
 /* boolean cookieDialog (in nsIDOMWindow parent, in nsICookie cookie, in ACString hostname, in long cookiesFromHost, in boolean changingCookie, inout boolean checkValue); */
-NS_IMETHODIMP GCookiePromptService::CookieDialog(nsIDOMWindow *aParent, nsICookie *aCookie, const nsACString & aHostname, PRInt32 aCookiesFromHost, PRBool aChangingCookie, PRBool *_checkValue, PRBool *_retval)
+NS_IMETHODIMP GCookiePromptService::CookieDialog(nsIDOMWindow *aParent, nsICookie *aCookie,
+						 const nsACString & aHostname, 
+						 PRInt32 aCookiesFromHost, 
+						 PRBool aChangingCookie, 
+						 PRBool *_checkValue, PRBool *_retval)
 {
-	// TODO short-circuit and accept session cookies as per preference
-	// TODO until mozilla starts supporting it natively?
-
 	GtkWidget *dialog, *button;
         gboolean checkValue;
 
-	GtkWidget *gparent = MozillaFindGtkParent (aParent);
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
+	GtkWidget *gparent = GaleonUtils::FindGtkParent (aParent);
 
 	dialog = hig_alert_new (GTK_WINDOW (gparent),
 				(GtkDialogFlags)NULL,
@@ -150,18 +131,24 @@ NS_IMETHODIMP GCookiePromptService::CookieDialog(nsIDOMWindow *aParent, nsICooki
 				NULL, NULL,
 				NULL);
 
-	button = create_button_with_image (_("_Reject"), GTK_STOCK_NO);
+	button = gul_gui_image_button_new (_("_Reject"), GTK_STOCK_NO);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_REJECT);
 
-	button = create_button_with_image (_("_Accept"), GTK_STOCK_YES);
+	button = gul_gui_image_button_new (_("_Accept"), GTK_STOCK_YES);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_ACCEPT);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 
+	GulCString hostname (aHostname);
+
 	// Set up the text
 	hig_alert_set_primary_printf (HIG_ALERT (dialog), _("Accept cookie from %s?"), 
-				      PromiseFlatCString(aHostname).get());
+				      hostname.get());
 
-	char *secondary = make_secondary_text (PromiseFlatCString(aHostname).get(), 
+	char *secondary = make_secondary_text (hostname.get(),
 			                       aChangingCookie, aCookiesFromHost);
 
 	hig_alert_set_secondary_printf (HIG_ALERT (dialog), "%s", secondary);
@@ -176,7 +163,7 @@ NS_IMETHODIMP GCookiePromptService::CookieDialog(nsIDOMWindow *aParent, nsICooki
 
         /* In Mozilla 1.4+ they changed the _checkValue from read/write to write only,
          * so mozilla does not store the state for us, we need to do it :( */
-	checkValue = gnome_config_get_bool (STATE_COOKIE_REMEMBER_DECISION "=false");
+	checkValue = gul_state_get_boolean (STATE_COOKIE_REMEMBER_DECISION, FALSE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(checkbutton), checkValue);
 
 	// add the expander button and the cookie details table
@@ -212,7 +199,7 @@ NS_IMETHODIMP GCookiePromptService::CookieDialog(nsIDOMWindow *aParent, nsICooki
 		*_checkValue = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbutton));
 
                 /* Save the value of this check box */
-		gnome_config_set_bool (STATE_COOKIE_REMEMBER_DECISION, *_checkValue);
+		gul_state_set_boolean (STATE_COOKIE_REMEMBER_DECISION, *_checkValue);
 	}
 	else
 	{
