@@ -19,6 +19,11 @@
 #include <config.h>
 #endif
 
+#include <nscore.h>
+#define MOZILLA_INTERNAL_API
+#include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
+
 #include "EventContext.h"
 #include "GaleonWrapper.h"
 #include "GaleonUtils.h"
@@ -54,13 +59,15 @@
 #include <nsIDOMEventTarget.h>
 #include <nsIDOMMouseEvent.h>
 #include <nsIDOMKeyEvent.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
 #include <nsITextToSubURI.h>
 #include <nsIWebBrowser.h>
 #include <nsISelection.h>
 #include <nsIDOM3Node.h>
+#include <nsIDOMNSUIEvent.h>
+#include <nsIDOMNSHTMLDocument.h>
+#include <nsIDOMHTMLSelectElement.h>
+#include <nsIDOMHTMLIsIndexElement.h>
+
 #include <nsMemory.h>
 
 #include <gdk/gdkkeysyms.h>
@@ -903,3 +910,90 @@ nsresult EventContext::ResolveBaseURL (const nsAString &relurl, nsACString &url)
 
 	return base->Resolve (GulCString (relurl), url);
 }
+
+PRBool
+EventContext::CheckKeyPress (nsIDOMKeyEvent *aEvent)
+{
+	PRBool retval = PR_FALSE;
+
+#ifdef HAVE_NSIDOMNSEVENT_GETISTRUSTED
+	/* make sure the event is trusted */
+	nsCOMPtr<nsIDOMNSEvent> nsEvent (do_QueryInterface (aEvent));
+	NS_ENSURE_TRUE (nsEvent, retval);
+	PRBool isTrusted = PR_FALSE;
+	nsEvent->GetIsTrusted (&isTrusted);
+	if (!isTrusted) return retval;
+#endif
+
+	/* check for alt/ctrl */
+	PRBool isCtrl = PR_FALSE, isAlt = PR_FALSE;
+	aEvent->GetCtrlKey (&isCtrl);
+	aEvent->GetAltKey (&isAlt);
+	if (isCtrl || isAlt) return retval;
+
+	nsCOMPtr<nsIDOMNSUIEvent> uiEvent (do_QueryInterface (aEvent));
+	NS_ENSURE_TRUE (uiEvent, retval);
+
+	/* check for already handled event */
+	PRBool isPrevented = PR_FALSE;
+	uiEvent->GetPreventDefault (&isPrevented);
+	if (isPrevented) return retval;
+
+	/* check for form controls */
+	nsresult rv;
+	nsCOMPtr<nsIDOMEventTarget> target;
+	rv = aEvent->GetTarget (getter_AddRefs (target));
+	NS_ENSURE_SUCCESS (rv, retval);
+
+	nsCOMPtr<nsIDOMHTMLInputElement> inputElement (do_QueryInterface (target));
+	if (inputElement)
+	{
+		nsEmbedString type;
+		inputElement->GetType (type);
+
+		nsEmbedCString (cType);
+		NS_UTF16ToCString (type, NS_CSTRING_ENCODING_UTF8, cType);
+
+		if (g_ascii_strcasecmp (cType.get(), "text") == 0 ||
+		    g_ascii_strcasecmp (cType.get(), "password") == 0 ||
+		    g_ascii_strcasecmp (cType.get(), "file") == 0) return retval;
+	}
+
+	nsCOMPtr<nsIDOMHTMLTextAreaElement> textArea;
+	nsCOMPtr<nsIDOMHTMLSelectElement> selectElement;
+	nsCOMPtr<nsIDOMHTMLIsIndexElement> indexElement;
+
+	if ((textArea = do_QueryInterface (target)) ||
+	    (selectElement = do_QueryInterface (target)) ||
+	    (indexElement = do_QueryInterface (target))) return retval;
+
+	/* check for design mode */
+	nsCOMPtr<nsIDOMNode> node (do_QueryInterface (target, &rv));
+	NS_ENSURE_SUCCESS (rv, PR_FALSE);
+
+	nsCOMPtr<nsIDOMDocument> doc;
+	rv = node->GetOwnerDocument (getter_AddRefs (doc));
+	NS_ENSURE_SUCCESS (rv, retval);
+
+	nsCOMPtr<nsIDOMXULDocument> xul_document (do_QueryInterface(doc, &rv));
+	if (xul_document) return retval;
+
+	nsCOMPtr<nsIDOMNSHTMLDocument> htmlDoc (do_QueryInterface (doc));
+	if (htmlDoc)
+	{
+		nsEmbedString uDesign;
+		rv = htmlDoc->GetDesignMode (uDesign);
+		NS_ENSURE_SUCCESS (rv, retval);
+
+		nsEmbedCString design;
+		NS_UTF16ToCString (uDesign, NS_CSTRING_ENCODING_UTF8, design);
+
+		retval = g_ascii_strcasecmp (design.get(), "on") != 0;
+	}
+	else
+	{
+		retval = PR_TRUE;
+	}
+
+	return retval;
+}
