@@ -16,7 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: FilePicker.h,v 1.6 2004/04/21 20:49:50 crispin Exp $
  */
 
 #ifndef GALEON_FILEPICKER_H
@@ -26,6 +25,7 @@
 #include <nsISupports.h>
 
 #include "gul-file-chooser.h"
+#include "GulString.h"
 
 #define G_FILEPICKER_CID			     \
 { /* 3636dc79-0b42-4bad-8a3f-ae15d3671d17 */         \
@@ -38,10 +38,6 @@
 #define G_FILEPICKER_CONTRACTID	"@mozilla.org/filepicker;1"
 #define G_FILEPICKER_CLASSNAME	"Galeon File Picker Implementation"
 
-class nsIFactory;
-
-extern nsresult NS_NewFilePickerFactory(nsIFactory** aFactory);
-
 class GFilePicker : public nsIFilePicker
 {
 public:
@@ -52,8 +48,11 @@ public:
 	virtual ~GFilePicker();
 
 private:
+	nsresult AppendFilter (const char * title, const char * pattern);
+
 	GulFileChooser *mDialog;
 	PRInt16 mMode;
+	GulString mDefaultString;
 };
 
 #endif
