@@ -19,24 +19,28 @@
 #ifndef GALEON_WRAPPER_H
 #define GALEON_WRAPPER_H
 
-#include "nsIDocShell.h"
-#include "nsIWebNavigation.h"
-#include "nsIWebPageDescriptor.h"
-#include "nsISHistory.h"
-#include "nsIWebBrowser.h"
-#include "nsIWebProgressListener.h"
-#include "nsCOMPtr.h"
-#include "nsIDOMEventReceiver.h"
-#include "nsIDOMDocument.h"
-#include "nsPIDOMWindow.h"
-#include "nsIDOMEventListener.h"
-#include "nsIChannel.h"
-#include "nsICacheEntryDescriptor.h"
-#include "nsIStyleSheet.h"
-#include "nsString.h"
+#include <nsIDOMEventListener.h>
+#include <nsIDOMContextMenuListener.h>
+#include <nsCOMPtr.h>
+#define MOZILLA_CLIENT
 #include <gtkmozembed.h>
+#undef MOZILLA_CLIENT
 
-#include "nsIPrintSettings.h"
+#include "galeon-embed.h"
+
+/* Forward declarations */
+class nsIDocShell;
+class nsIWebNavigation;
+class nsIWebPageDescriptor;
+class nsISHistory;
+class nsIWebBrowser;
+class nsIChannel;
+class nsICacheEntryDescriptor;
+class nsIStyleSheet;
+class nsIContentViewer;
+class nsIPrintSettings;
+class nsISecureBrowserUI;
+class nsACString;
 
 #ifdef HAVE_MOZILLA_PSM
 class nsISSLStatus;
@@ -66,6 +70,29 @@ class GDOMPopupEventListener : public GDOMEventListener
   	virtual ~GDOMPopupEventListener() {}
 };
 
+class GDOMModalAlertEventListener : public GDOMEventListener
+{
+public:
+	NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent);
+};
+
+
+class GDOMContextMenuListener : public nsIDOMContextMenuListener
+{
+ public:
+  	NS_DECL_ISUPPORTS
+	GDOMContextMenuListener() {}	
+	virtual ~GDOMContextMenuListener() {};
+
+	nsresult Init(GaleonEmbed* aEmbed, PRBool aIsCapturePhase);
+
+	NS_IMETHOD ContextMenu(nsIDOMEvent* aContextMenuEvent);
+	NS_IMETHOD HandleEvent(nsIDOMEvent*);
+  protected:
+	PRBool          mIsCapturePhase;
+	GaleonEmbed	*mEmbed;
+};
+
 class GaleonWrapper
 {
   public:
@@ -75,10 +102,10 @@ class GaleonWrapper
 	nsresult Init (GtkMozEmbed *mozembed);
 	nsresult Destroy (void);
 
-	nsresult SetZoom (float aTextZoom, PRBool reflow);
+	nsresult SetZoom (float aTextZoom);
 	nsresult GetZoom (float *aTextZoom);
 
-	nsresult Print (nsIPrintSettings *options, PRBool preview);
+	nsresult Print (nsIPrintSettings *options, PRBool preview, GtkWindow *parent);
 	nsresult GetPrintSettings (nsIPrintSettings * *options);
 	nsresult PrintPreviewClose (void);
 	nsresult PrintPreviewNumPages (int *numPages);
@@ -90,16 +117,20 @@ class GaleonWrapper
 
 	nsresult Find (PRBool backwards, PRBool *didFind);
 
-	nsresult GetMainDocumentUrl (nsCString &url);
-	nsresult GetDocumentUrl (nsCString &url);
+	nsresult GetMainDocumentUrl (nsACString &url);
+	nsresult GetDocumentUrl (nsACString &url);
+	nsresult FocusActivate ();
+	nsresult FocusDeactivate ();
+
+	nsresult ReloadViewSourcePage ();
+	nsresult ReloadFrame ();
 
-	nsresult ReloadDocument ();
 	nsresult LoadDocument(nsISupports *aPageDescriptor, PRUint32 aDisplayType);
 	nsresult GetPageDescriptor(nsISupports **aPageDescriptor);
 
 	nsresult GetSHInfo (PRInt32 *count, PRInt32 *index);
 	nsresult GetSHTitleAtIndex (PRInt32 index, PRUnichar **title);
-	nsresult GetSHUrlAtIndex (PRInt32 index, nsCString &url);
+	nsresult GetSHUrlAtIndex (PRInt32 index, nsACString &url);
 
 	nsresult CopyHistoryTo (GaleonWrapper *embed, PRBool back_history,
 				PRBool forward_history, PRBool set_current);
@@ -108,9 +139,11 @@ class GaleonWrapper
 
 	nsresult ClearHistory();
 
-	nsresult ForceEncoding (const char *encoding);
-	
-	nsresult GetEncodingInfo (GaleonEncodingPageInfo **info);
+	nsresult SetForcedEncoding (const char *encoding);
+
+	nsresult GetForcedEncoding (nsACString &encoding);
+
+	nsresult GetEncoding (nsACString &encoding);
 	
 	nsresult GetHasModifiedForms (PRBool *modified);
 
@@ -136,7 +169,7 @@ class GaleonWrapper
 
 	nsresult GetLinkInterfaceItems (GList **list);
 
-	nsresult GetRealURL (nsCString &ret);
+	nsresult GetRealURL (nsACString &ret);
 
 	nsresult SelectAll (void);
 
@@ -145,38 +178,42 @@ class GaleonWrapper
 	nsresult ScrollLeft (void);
 	nsresult ScrollRight (void);
 
+	nsresult ScrollPageUp (void);
+	nsresult ScrollPageDown (void);
+
 	nsresult FineScroll (int horiz, int vert);
 
 	nsresult GetPageProperties (EmbedPageProperties *props);
 	nsresult ShowPageCertificate ();
 
-	nsresult GetSecurityTooltip (char **tooltip);
+	nsresult GetSecurityInfo (PRUint32 *aState, nsACString &aDescription);
 
-	nsresult EvaluateJS (char *script);
+	nsresult EvaluateJS (const char *script);
 
-	nsresult PushTargetDocument (nsIDOMDocument *domDoc);
+	nsresult PushTargetDocument (nsIDOMDocument *domDoc, GaleonEmbedEvent *event);
 	nsresult PopTargetDocument ();
 
 	nsresult GetDOMDocument (nsIDOMDocument **aDOMDocument);
 	nsresult GetDOMWindow (nsIDOMWindow **aDOMWindow);
 
-	nsresult SetChannel (nsIChannel *channel);
-
 	nsCOMPtr<nsIWebBrowser>           mWebBrowser;
   private:
 
 	nsCOMPtr<nsIDOMDocument> mTargetDocument;
-	nsCOMPtr<nsIChannel> mChannel;
+	nsCOMPtr<nsISecureBrowserUI> mSecurityInfo;
 
 	nsCOMPtr<nsIDOMEventTarget> mEventTarget;
 	nsCOMPtr<GDOMEventListener> mFaviconEventListener;
 	nsCOMPtr<GDOMEventListener> mPopupEventListener;
+	nsCOMPtr<GDOMContextMenuListener> mContextMenuListenerBubble;
+	nsCOMPtr<GDOMContextMenuListener> mContextMenuListenerCapture;
+	nsCOMPtr<GDOMModalAlertEventListener> mModalAlertListener;
 
-	nsresult SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell);
-	nsresult GetDocShell (nsIDocShell **aDocShell);
+	GaleonEmbedEvent *mCurrentEvent;
+
+	nsresult GetContentViewer (nsIContentViewer **aViewer);
 	nsresult GetFocusedDOMWindow (nsIDOMWindow **aDOMWindow);
 	nsresult GetSHistory (nsISHistory **aSHistory);
-	nsresult GetWebNavigation(nsIWebNavigation **aWebNavigation);
 
 #ifdef HAVE_MOZILLA_PSM
 	nsresult GetSSLStatus (nsISSLStatus **aSSLStatus);
