@@ -27,6 +27,7 @@
 #define MOZILLA_STRICT_API
 
 #include "GaleonUtils.h"
+#include "AutoJSContextStack.h"
 
 #include <nsIServiceManager.h>
 #include <nsIInterfaceRequestor.h>
@@ -143,6 +144,11 @@ GtkNSSClientAuthDialogs::ChooseCertificate (nsIInterfaceRequestor *ctx,
 	char *msg, *tt_cn, *markup_text;
 	PRUint32 i;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
