@@ -20,11 +20,19 @@
 #include "config.h"
 #endif
 
+#include <nscore.h>
+#define MOZILLA_INTERNAL_API
+#include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
+
+#define MOZILLA_CLIENT
+#include "gtkmozembed.h"
+#include "gtkmozembed_internal.h"
+#undef MOZILLA_CLIENT
 #include "glib.h"
 #include "js-console.h"
 #include "gul-string.h"
 #include "galeon-config.h"
-#include "gtkmozembed.h"
 #include "mozilla-embed-shell.h"
 #include "mozilla-prefs.h"
 #include "prefs-strings.h"
@@ -50,9 +58,6 @@
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
 #include <nsNetCID.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
 #include <nsIIOService.h>
 #include <nsIJVMManager.h>
 #include <nsIFontEnumerator.h>
@@ -77,6 +82,9 @@
 # include <chrome/nsIChromeRegistrySea.h>
 #endif
 
+#ifdef HAVE_NSITOOLKITCHROMEREGISTRY_H
+#include "EphyDirectoryProvider.h"
+#endif
 
 #define MOZILLA_PROFILE_DIR  GALEON_DIR"/mozilla"
 #define MOZILLA_PROFILE_NAME "galeon"
@@ -186,6 +194,9 @@ mozilla_init_home (void)
 	}
 
         gtk_moz_embed_set_comp_path (mozilla_home);
+#ifdef HAVE_GTK_MOZ_EMBED_SET_PATH
+	gtk_moz_embed_set_path (mozilla_home);
+#endif
 	g_free (mozilla_home);
 }
 
@@ -362,7 +373,13 @@ mozilla_embed_shell_init (MozillaEmbedShell *mes)
 	mozilla_init_plugin_path();
 	mozilla_init_home ();
 	mozilla_init_profile ();
-	
+
+#if HAVE_NSITOOLKITCHROMEREGISTRY_H
+	nsCOMPtr<nsIDirectoryServiceProvider> dp = new EphyDirectoryProvider ();
+	if (dp) {
+		gtk_moz_embed_set_directory_service_provider (dp);
+	}
+#endif
 	/* Fire up the best */
 	gtk_moz_embed_push_startup ();
 }
