@@ -32,7 +32,6 @@
 #include "galeon-embed-utils.h"
 #include "galeon-css-menu.h"
 
-#include "gul-gobject-misc.h"
 #include "galeon-marshal.h"
 #include "galeon-debug.h"
 #include "gul-string.h"
@@ -41,6 +40,10 @@
 /**
  * Private data
  */
+#define GALEON_CSS_MENU_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GALEON_TYPE_CSS_MENU, GaleonCssMenuPrivate))
+
+
 struct _GaleonCssMenuPrivate
 {
 	GaleonWindow *window;
@@ -63,15 +66,18 @@ enum
 	PROP_WINDOW
 };
 
-static void galeon_css_menu_rebuild (GaleonCssMenu *menu);
 
-static gpointer g_object_class;
+G_DEFINE_TYPE (GaleonCssMenu, galeon_css_menu, G_TYPE_OBJECT);
+
+static void galeon_css_menu_rebuild (GaleonCssMenu *menu);
 
 #define CSS_MENU_PLACEHOLDER_PATH       "/menubar/View/ViewStylesMenuPlaceholder"
 #define STYLESHEET_KEY                  "galeon-stylesheet"
 #define USERSTYLE_KEY			"galeon-user-style"
 #define REBUILD_TIMEOUT 700
 
+#define MENU_ITEM_MAX_LENGTH 40
+
 static void
 activate_stylesheet_cb (GtkAction *action, GaleonCssMenu *menu)
 {
@@ -80,7 +86,7 @@ activate_stylesheet_cb (GtkAction *action, GaleonCssMenu *menu)
 
 	if (menu->priv->block_signals) return;
 
-	g_return_if_fail (IS_GALEON_EMBED (p->embed));
+	g_return_if_fail (GALEON_IS_EMBED (p->embed));
 
 	style = g_object_get_data (G_OBJECT (action), STYLESHEET_KEY);
 
@@ -118,12 +124,16 @@ activate_stylesheet_cb (GtkAction *action, GaleonCssMenu *menu)
 }
 
 static GtkAction *
-create_stylesheet_action (GaleonCssMenu *menu, EmbedStyleSheet *style)
+create_stylesheet_action (GaleonCssMenu *menu, EmbedStyleSheet *style, int num)
 {
 	GtkAction  *action;
-	gchar *name, *tooltip, *label;
+	gchar *tooltip, *label, *shortened;
+	gchar name[40];
 
-	label = gul_string_double_underscores (style->name);
+	g_snprintf (name, sizeof (name), "GaleonCSS%dAction", num);
+
+	shortened = gul_string_shorten (style->name, MENU_ITEM_MAX_LENGTH);
+	label = gul_string_double_underscores (shortened);
 
 	switch (style->type)
 	{
@@ -146,7 +156,6 @@ create_stylesheet_action (GaleonCssMenu *menu, EmbedStyleSheet *style)
 		break;
 	}
 
-	name = g_strdup_printf ("GaleonCSS%sAction", style->name);
 	action = g_object_new (GTK_TYPE_RADIO_ACTION,
 			       "name", name,
 			       "label", label,
@@ -163,7 +172,6 @@ create_stylesheet_action (GaleonCssMenu *menu, EmbedStyleSheet *style)
 	gtk_action_group_add_action (menu->priv->item_action_group, action);
 	g_object_unref (action);
 
-	g_free (name);
 	g_free (label);
 	g_free (tooltip);
 	return action;
@@ -173,11 +181,11 @@ static void
 galeon_css_menu_rebuild (GaleonCssMenu *menu)
 {
 	GaleonCssMenuPrivate *p = menu->priv;
-	GList *stylesheets = 0, *user_styles = 0, *l;
+	GList *stylesheets = NULL, *user_styles = NULL, *l;
 	GtkAction *action;
-	gresult result;
 	EmbedStyleSheet *current = NULL, *user_current = NULL;
 	GSList *radio_group = NULL, *user_group = NULL;
+	int count = 0;
 
 	if (p->merge_id > 0)
 	{
@@ -190,16 +198,14 @@ galeon_css_menu_rebuild (GaleonCssMenu *menu)
 	{
 		gtk_ui_manager_remove_action_group (p->merge, p->item_action_group);
 		g_object_unref (p->item_action_group);
-		p->item_action_group = 0;
+		p->item_action_group = NULL;
 	}
 
 	LOG ("Rebuilding stylesheet menu");
 
-	result = galeon_embed_get_stylesheets (p->embed, &stylesheets);
-	if (result != G_OK) stylesheets = NULL;
+	stylesheets = galeon_embed_get_stylesheets (p->embed);
 
-	result = galeon_embed_get_selected_stylesheet (p->embed, &current);
-	if (result != G_OK) current = NULL;
+	current = galeon_embed_get_selected_stylesheet (p->embed);
 
 	user_styles = galeon_embed_utils_get_user_stylesheets();
 
@@ -222,15 +228,13 @@ galeon_css_menu_rebuild (GaleonCssMenu *menu)
 	
 	for (l = stylesheets; l ; l = l->next)
 	{
-		gchar name[128];
 		EmbedStyleSheet *style = (EmbedStyleSheet*) (l->data);
-		action = create_stylesheet_action (menu, style);
-
-		g_snprintf (name, sizeof(name), "%sItem", gtk_action_get_name (action));
+		action = create_stylesheet_action (menu, style, count++);
 
 		gtk_ui_manager_add_ui (p->merge, p->merge_id,
 				       CSS_MENU_PLACEHOLDER_PATH "/StyleMenu",
-				       name, gtk_action_get_name (action),
+				       gtk_action_get_name (action),
+				       gtk_action_get_name (action),
 				       GTK_UI_MANAGER_MENUITEM, FALSE);
 
 		/* Make sure all widgets are in the same radio_group */
@@ -251,15 +255,13 @@ galeon_css_menu_rebuild (GaleonCssMenu *menu)
 
 	for (l = user_styles; l ; l = l->next)
 	{
-		gchar name[128];
 		EmbedStyleSheet *style = (EmbedStyleSheet*)(l->data);
-		action = create_stylesheet_action(menu, style);
-
-		g_snprintf(name, sizeof(name), "%sItem", gtk_action_get_name(action));
+		action = create_stylesheet_action(menu, style, count++);
 
 		gtk_ui_manager_add_ui(p->merge, p->merge_id,
 				      CSS_MENU_PLACEHOLDER_PATH "/StyleMenu",
-				      name, gtk_action_get_name(action),
+				      gtk_action_get_name(action),
+				      gtk_action_get_name(action),
 				      GTK_UI_MANAGER_MENUITEM, FALSE);
 
 		/* Make sure all widgets are in the same radio_gruop */
@@ -482,15 +484,14 @@ galeon_css_menu_finalize_impl(GObject *o)
 
 	gtk_ui_manager_ensure_update (p->merge);
 	g_object_unref (p->merge);
-	g_free(p);
 
-	G_OBJECT_CLASS(g_object_class)->finalize (o);
+	G_OBJECT_CLASS(galeon_css_menu_parent_class)->finalize (o);
 }
 
 static void 
 galeon_css_menu_init(GaleonCssMenu *m)
 {
-	GaleonCssMenuPrivate *p = g_new0 (GaleonCssMenuPrivate, 1);
+	GaleonCssMenuPrivate *p = GALEON_CSS_MENU_GET_PRIVATE (m);
 	m->priv = p;
 }
 
@@ -498,7 +499,6 @@ static void
 galeon_css_menu_class_init(GaleonCssMenuClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	g_object_class = g_type_class_peek_parent(klass);
 
 	object_class->set_property = galeon_css_menu_set_property;
 	object_class->get_property = galeon_css_menu_get_property;
@@ -506,24 +506,21 @@ galeon_css_menu_class_init(GaleonCssMenuClass *klass)
 
 	g_object_class_install_property (object_class,
                                          PROP_WINDOW,
-                                         g_param_spec_object ("GaleonWindow",
-                                                              "GaleonWindow",
+                                         g_param_spec_object ("window",
+                                                              "window",
                                                               "Parent window",
-                                                              GALEON_WINDOW_TYPE,
+                                                              GALEON_TYPE_WINDOW,
                                                               G_PARAM_READWRITE |
 							      G_PARAM_CONSTRUCT_ONLY));
+
+	g_type_class_add_private (klass, sizeof (GaleonCssMenuPrivate));
 }
 
 
-MAKE_GET_TYPE(galeon_css_menu, "GaleonCssMenu", GaleonCssMenu,
-	      galeon_css_menu_class_init,
-	      galeon_css_menu_init, G_TYPE_OBJECT);
-
-
 GaleonCssMenu *
 galeon_css_menu_new (GaleonWindow *window)
 {
 	GaleonCssMenu *ret = g_object_new (GALEON_TYPE_CSS_MENU, 
-					   "GaleonWindow", window, NULL);
+					   "window", window, NULL);
 	return ret;
 }
