@@ -22,36 +22,22 @@
 
 #include <gtk/gtkwidget.h>
 #include <gtk/gtktoolbar.h>
-#include <string.h>
+#include <gtk/gtkstock.h>
 
 #include "galeon-favicon.h"
 #include "galeon-shell.h"
-#include "galeon-shell.h"
-#include "pixbuf-cache.h"
 
-static void galeon_favicon_class_init (GaleonFaviconClass *klass);
-static void galeon_favicon_init (GaleonFavicon *ma);
-static void galeon_favicon_finalize (GObject *object);
-static void galeon_favicon_set_property (GObject *object,
-					 guint prop_id,
-					 const GValue *value,
-					 GParamSpec *pspec);
-static void galeon_favicon_get_property (GObject *object,
-					 guint prop_id,
-					 GValue *value,
-					 GParamSpec *pspec);
-static void galeon_favicon_update_image (GaleonFavicon *favicon);
-static void cache_changed_cb (GaleonFaviconCache *cache,
-		              const char *url,
-		              GaleonFavicon *favicon);
+
+#define GALEON_FAVICON_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GALEON_TYPE_FAVICON, GaleonFaviconPrivate))
+
 
 struct GaleonFaviconPrivate
 {
 	GaleonFaviconCache *cache;
-	gboolean isdefault;
+	GaleonFaviconCacheEntry *entry;
 
 	char *url;
-	GdkPixbuf *default_pixbuf;
 };
 
 enum
@@ -69,116 +55,103 @@ enum
 
 static guint galeon_favicon_signals[LAST_SIGNAL] = { 0 };
 
-static GObjectClass *parent_class = NULL;
+G_DEFINE_TYPE (GaleonFavicon, galeon_favicon, GTK_TYPE_IMAGE);
 
-GType
-galeon_favicon_get_type (void)
+static void
+galeon_favicon_update_image (GaleonFavicon *favicon)
 {
-	static GType galeon_favicon_type = 0;
+	GdkPixbuf *pixbuf = NULL;
 
-	if (galeon_favicon_type == 0)
+	if (favicon->priv->entry)
 	{
-		static const GTypeInfo our_info =
+		pixbuf = galeon_favicon_cache_entry_get_pixbuf (favicon->priv->entry);
+	}
+
+	if (!pixbuf)
+	{
+		gtk_image_set_from_stock (GTK_IMAGE (favicon),
+					  GTK_STOCK_JUMP_TO,
+					  GTK_ICON_SIZE_MENU);
+	}
+	else
+	{
+		gtk_image_set_from_pixbuf (GTK_IMAGE (favicon), pixbuf);
+	}
+
+	g_signal_emit (G_OBJECT (favicon), galeon_favicon_signals[CHANGED], 0);
+}	
+
+static void
+cache_image_changed_cb (GaleonFaviconCacheEntry *entry, GParamSpec   *pspec,
+			GaleonFavicon* favicon)
+{
+	galeon_favicon_update_image (favicon);	
+}
+
+void
+galeon_favicon_set_url (GaleonFavicon *favicon,
+		        const char *url)
+{
+	g_return_if_fail (GALEON_IS_FAVICON (favicon));
+
+	g_free (favicon->priv->url);
+	favicon->priv->url = g_strdup (url);
+
+	if (favicon->priv->entry)
+	{
+		g_signal_handlers_disconnect_by_func (favicon->priv->entry, 
+						      G_CALLBACK (cache_image_changed_cb),
+						      favicon);
+
+		g_object_unref (favicon->priv->entry);
+		favicon->priv->entry = NULL;
+	}
+
+	if (favicon->priv->url)
+	{
+		favicon->priv->entry = 
+			galeon_favicon_cache_lookup (favicon->priv->cache,
+						     favicon->priv->url);
+
+		if (favicon->priv->entry)
 		{
-			sizeof (GaleonFaviconClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) galeon_favicon_class_init,
-			NULL,
-			NULL,
-			sizeof (GaleonFavicon),
-			0,
-			(GInstanceInitFunc) galeon_favicon_init
-		};
+			g_signal_connect (favicon->priv->entry,
+					  "notify::pixbuf",
+					  G_CALLBACK (cache_image_changed_cb),
+					  favicon);
+		}
 
-		galeon_favicon_type = g_type_register_static (GTK_TYPE_IMAGE,
-							      "GaleonFavicon",
-							      &our_info, 0);
 	}
 
-	return galeon_favicon_type;
+	g_object_notify (G_OBJECT (favicon), "url");
+	galeon_favicon_update_image (favicon);
 }
 
-static void
-galeon_favicon_class_init (GaleonFaviconClass *klass)
+char *
+galeon_favicon_get_url (GaleonFavicon *favicon)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-	parent_class = g_type_class_peek_parent (klass);
-
-	object_class->finalize = galeon_favicon_finalize;
-
-	object_class->set_property = galeon_favicon_set_property;
-	object_class->get_property = galeon_favicon_get_property;
-
-	g_object_class_install_property (object_class,
-					 PROP_CACHE,
-					 g_param_spec_object ("cache",
-							      "Favicon cache",
-							      "Favicon cache",
-							      GALEON_TYPE_FAVICON_CACHE,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (object_class,
-					 PROP_URL,
-					 g_param_spec_string ("url",
-							      "Associated URL",
-							      "Associated URL",
-							      NULL,
-							      G_PARAM_READWRITE));
-
-	galeon_favicon_signals[CHANGED] =
-		g_signal_new ("changed",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GaleonFaviconClass, changed),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
+	g_return_val_if_fail (GALEON_IS_FAVICON (favicon), NULL);
+	
+	return g_strdup (favicon->priv->url);
 }
 
-static void
-galeon_favicon_init (GaleonFavicon *ma)
+gboolean
+galeon_favicon_get_is_default (GaleonFavicon *favicon)
 {
-	GtkWidget *dummy;
+	g_return_val_if_fail (GALEON_IS_FAVICON (favicon), FALSE);
 
-	ma->priv = g_new0 (GaleonFaviconPrivate, 1);
-	ma->priv->isdefault = TRUE;
-
-	dummy = gtk_toolbar_new ();
-	g_object_ref (dummy);
-	gtk_object_sink (GTK_OBJECT (dummy));
-
-	ma->priv->default_pixbuf = gtk_widget_render_icon (dummy,
-							   GALEON_STOCK_BOOKMARK,
-							   GTK_ICON_SIZE_MENU, NULL);
-	g_object_unref (dummy);
-
-	gtk_widget_set_size_request (GTK_WIDGET (ma), 16, 16);
-}
-
-static void
-galeon_favicon_finalize (GObject *object)
-{
-	GaleonFavicon *ma;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (GALEON_IS_FAVICON (object));
-
-	ma = GALEON_FAVICON (object);
-
-	g_return_if_fail (ma->priv != NULL);
-
-	g_free (ma->priv->url);
-
-	if (ma->priv->default_pixbuf)
+	if (favicon->priv->entry &&
+	    galeon_favicon_cache_entry_get_pixbuf (favicon->priv->entry))
 	{
-		g_object_unref (G_OBJECT (ma->priv->default_pixbuf));
+		return FALSE;
 	}
+	return TRUE;
+}
 
-	g_free (ma->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+static void
+galeon_favicon_set_cache (GaleonFavicon *favicon, GaleonFaviconCache *cache)
+{
+	favicon->priv->cache = cache;
 }
 
 static void
@@ -192,15 +165,7 @@ galeon_favicon_set_property (GObject *object,
 	switch (prop_id)
 	{
 	case PROP_CACHE:
-		favicon->priv->cache = g_value_get_object (value);
-
-		g_signal_connect_object (G_OBJECT (favicon->priv->cache),
-				         "changed",
-				         G_CALLBACK (cache_changed_cb),
-				         favicon,
-				         0);
-
-		galeon_favicon_update_image (favicon);
+		galeon_favicon_set_cache (favicon, g_value_get_object (value));
 		break;
 	case PROP_URL:
 		galeon_favicon_set_url (favicon, g_value_get_string (value));
@@ -233,82 +198,84 @@ galeon_favicon_get_property (GObject *object,
 	}
 }
 
+static void
+galeon_favicon_finalize (GObject *object)
+{
+	GaleonFavicon *favicon = GALEON_FAVICON (object);
+
+	g_free (favicon->priv->url);
+
+	if (favicon->priv->entry)
+	{
+		g_signal_handlers_disconnect_by_func (favicon->priv->entry, 
+						      G_CALLBACK (cache_image_changed_cb),
+						      favicon);
+
+		g_object_unref (favicon->priv->entry);
+	}
+
+	G_OBJECT_CLASS (galeon_favicon_parent_class)->finalize (object);
+}
+
+static void
+galeon_favicon_init (GaleonFavicon *ma)
+{
+	ma->priv = GALEON_FAVICON_GET_PRIVATE (ma);
+
+	gtk_widget_set_size_request (GTK_WIDGET (ma), 16, 16);
+}
+
+static void
+galeon_favicon_class_init (GaleonFaviconClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+	object_class->finalize = galeon_favicon_finalize;
+
+	object_class->set_property = galeon_favicon_set_property;
+	object_class->get_property = galeon_favicon_get_property;
+
+	g_object_class_install_property (object_class,
+					 PROP_CACHE,
+					 g_param_spec_object ("cache",
+							      "Favicon cache",
+							      "Favicon cache",
+							      GALEON_TYPE_FAVICON_CACHE,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_URL,
+					 g_param_spec_string ("url",
+							      "Associated URL",
+							      "Associated URL",
+							      NULL,
+							      G_PARAM_READWRITE));
+
+	galeon_favicon_signals[CHANGED] =
+		g_signal_new ("changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GaleonFaviconClass, changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
+	g_type_class_add_private (klass, sizeof (GaleonFaviconPrivate));
+}
+
+
 GtkWidget *
 galeon_favicon_new (const char *url)
 {
-	GaleonFavicon *favicon;
+	GObject *favicon;
 	GaleonFaviconCache *cache = galeon_shell_get_favicon_cache (galeon_shell);
 
 	g_return_val_if_fail (GALEON_IS_FAVICON_CACHE (cache), NULL);
 
-	favicon = GALEON_FAVICON (g_object_new (GALEON_TYPE_FAVICON,
-						"cache", cache,
-						"url", url,
-						NULL));
-
-	g_return_val_if_fail (favicon->priv != NULL, NULL);
+	favicon = g_object_new (GALEON_TYPE_FAVICON,
+				"cache", cache,
+				"url", url,
+				NULL);
 
 	return GTK_WIDGET (favicon);
 }
-
-void
-galeon_favicon_set_url (GaleonFavicon *favicon,
-		        const char *url)
-{
-	g_return_if_fail (GALEON_IS_FAVICON (favicon));
-
-	g_free (favicon->priv->url);
-	favicon->priv->url = g_strdup (url);
-
-	g_object_notify (G_OBJECT (favicon), "url");
-	galeon_favicon_update_image (favicon);
-}
-
-char *
-galeon_favicon_get_url (GaleonFavicon *favicon)
-{
-	g_return_val_if_fail (GALEON_IS_FAVICON (favicon), NULL);
-	
-	return g_strdup (favicon->priv->url);
-}
-
-gboolean
-galeon_favicon_get_is_default (GaleonFavicon *favicon)
-{
-	return favicon->priv->isdefault;
-}
-
-static void
-cache_changed_cb (GaleonFaviconCache *cache,
-		  const char *url,
-		  GaleonFavicon *favicon)
-{
-	if (galeon_favicon_cache_url_equal (url, favicon->priv->url))
-		galeon_favicon_update_image (favicon);
-}
-
-static void
-galeon_favicon_update_image (GaleonFavicon *favicon)
-{
-	GdkPixbuf *pixbuf;
-
-	g_return_if_fail (GALEON_IS_FAVICON_CACHE (favicon->priv->cache));
-
-	pixbuf = galeon_favicon_cache_lookup (favicon->priv->cache,
-					      favicon->priv->url);
-
-	favicon->priv->isdefault = (pixbuf == NULL);
-	if (!pixbuf)
-	{
-		pixbuf = favicon->priv->default_pixbuf;
-		favicon->priv->isdefault = TRUE;
-	}
-	else
-	{
-		favicon->priv->isdefault = FALSE;
-	}
-
-	gtk_image_set_from_pixbuf (GTK_IMAGE (favicon), pixbuf);
-
-	g_signal_emit (G_OBJECT (favicon), galeon_favicon_signals[CHANGED], 0);
-}	
