@@ -57,7 +57,7 @@ GtkWidget  *galeon_favicon_new      (const char *url);
 void        galeon_favicon_set_url  (GaleonFavicon *favicon,
 				     const char *url);
 
-const char *galeon_favicon_get_url  (GaleonFavicon *favicon);
+char *      galeon_favicon_get_url  (GaleonFavicon *favicon);
 
 
 gboolean    galeon_favicon_get_is_default (GaleonFavicon *favicon);
