@@ -31,12 +31,10 @@
 #include <gtk/gtkcombobox.h>
 
 #include "gul-glade.h"
-#include "gul-gobject-misc.h"
 #include "galeon-marshal.h"
 
 #include <string.h>
 
-static GtkDialogClass *parent_class;
 
 enum {
 	CHANGED,	/* indicate language selection changed */
@@ -46,12 +44,16 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+
+G_DEFINE_TYPE(GaleonLanguageEditor, galeon_language_editor, GTK_TYPE_DIALOG)
+
 /**
  * galeon_language_editor_set_available:
  * @languages: an array of #GulGuiOption items
  * @n_languages: number of items in @languages
  *
- * Sets the available languages and their titles.
+ * Sets the available languages and their titles, the titles are expected to
+ * already be translated.
  */
 void
 galeon_language_editor_set_available (GaleonLanguageEditor *self,
@@ -59,7 +61,7 @@ galeon_language_editor_set_available (GaleonLanguageEditor *self,
 				      guint                 n_languages)
 {
 	gul_gui_option_combobox_populate(GTK_COMBO_BOX(self->combobox),
-					 languages, n_languages, TRUE);
+					 languages, n_languages, TRUE, FALSE);
 
 	/* 
 	 * Adding rows produces a warning if this is not done. *sigh*
@@ -98,7 +100,7 @@ galeon_language_editor_set_selected (GaleonLanguageEditor *self, GSList *values)
 					   OPTION_COL_TITLE, &iter_title,
 					   OPTION_COL_VALUE, &iter_value,
 					   -1);
-			if (strcmp(value, iter_value) == 0)
+			if (g_ascii_strcasecmp(value, iter_value) == 0)
 			{
 				found = TRUE;
 				title = iter_title;
@@ -225,6 +227,8 @@ treeview_construct (GtkWidget *treeview)
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW(treeview),
 			         GTK_TREE_MODEL(store));
+
+	g_object_unref (store);
 }
 
 static void
@@ -391,7 +395,7 @@ static void
 galeon_language_editor_init (GaleonLanguageEditor *self)
 {
 	GladeXML         *gxml;
-	GtkWidget        *content;
+	GtkWidget        *content, *glade_dialog;
 	GtkTreeSelection *selection;
 
 	gxml = gul_glade_widget_new ("prefs-dialog.glade", "language_dialog", NULL, self);
@@ -405,6 +409,7 @@ galeon_language_editor_init (GaleonLanguageEditor *self)
 			"remove_button", &self->remove,
 			"up_button",     &self->up,
 			"down_button",   &self->down,
+			"language_dialog", &glade_dialog,
 			NULL);
 
 	gtk_widget_reparent (content, GTK_DIALOG(self)->vbox);
@@ -421,12 +426,13 @@ galeon_language_editor_init (GaleonLanguageEditor *self)
 	g_signal_connect (G_OBJECT(self->down),   "clicked", G_CALLBACK(on_down_clicked),   self);
 
 	g_object_unref (gxml);
+	gtk_widget_destroy (glade_dialog);
 }
 
 static void
 galeon_language_editor_finalize (GObject *object)
 {
-	G_OBJECT_CLASS(parent_class)->finalize (object);
+	G_OBJECT_CLASS(galeon_language_editor_parent_class)->finalize (object);
 }
 
 static void
@@ -434,7 +440,6 @@ galeon_language_editor_class_init (GaleonLanguageEditorClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 
-	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = galeon_language_editor_finalize;
 
@@ -458,4 +463,3 @@ galeon_language_editor_class_init (GaleonLanguageEditorClass *klass)
 			G_TYPE_STRING);
 }
 
-GUL_TYPE_FUNC(galeon_language_editor, GaleonLanguageEditor, GTK_TYPE_DIALOG)
