@@ -20,6 +20,9 @@
 #include <config.h>
 #endif
 
+/* We use the deprecated popt stuff from libgnome .. */
+#undef GNOME_DISABLE_DEPRECATED
+
 #include "galeon-shell.h"
 #include "galeon-automation.h"
 #include "galeon-window.h"
@@ -180,7 +183,7 @@ slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
 		XChangeProperty (xdisplay, 
 				 xwindow, atom_name,
 				 atom_type,
-				 8, PropModeReplace, name, strlen (name));
+				 8, PropModeReplace, (guchar*)name, strlen (name));
 	}
 	
 	XWindowEvent (xdisplay,
