@@ -36,6 +36,7 @@
 #include "galeon-debug.h"
 #include "gul-x11.h"
 #include "gul-gui.h"
+#include "galeon-simple-window.h"
 
 #include <libgnomeui/gnome-client.h>
 #include <bonobo/bonobo-main.h>
@@ -152,29 +153,11 @@ galeon_shell_new_window_cb (GaleonEmbedShell *shell,
                             EmbedChromeMask chromemask,
 			    gpointer data)
 {
-	GaleonTab *new_tab;
-        GaleonWindow *window;
-	gboolean open_in_tab;
+	GaleonSimpleWindow *window;
 
-	open_in_tab = (chromemask & EMBED_CHROME_OPENASCHROME) ?
-		      FALSE :
-		      eel_gconf_get_boolean (CONF_TABS_TABBED_POPUPS);
+	window = galeon_simple_window_new ();
 
-	if (open_in_tab)
-	{
-		window = galeon_shell_get_active_window (galeon_shell);
-	}
-	else
-	{
-		window = galeon_window_new ();
-		galeon_window_set_chrome (window, chromemask);
-	}
-	
-	new_tab = galeon_tab_new ();
-	galeon_window_add_tab (window, new_tab,
-			       GALEON_WINDOW_ADD_TAB_LAST, FALSE);
-	
-	return galeon_tab_get_embed (new_tab);
+	return galeon_simple_window_get_embed (window);
 }
 
 
