@@ -36,12 +36,20 @@
 #include "gul-state.h"
 #include "gul-gui.h"
 #include "gul-file-chooser.h"
+#include "galeon-find-toolbar.h"
+#include "galeon-simple-window.h"
 
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-about.h>
+#include <gtk/gtkversion.h>
+#if GTK_CHECK_VERSION(2,5,0)
+# include <gtk/gtkaboutdialog.h>
+# define USE_GTKABOUTDIALOG
+#else
+# include <libgnomeui/gnome-about.h>
+#endif
 #include <gtk/gtkeditable.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkaction.h>
@@ -59,13 +67,6 @@ enum
 };
 
 
-void
-window_cmd_edit_find (GtkAction *action, 
-		      GaleonWindow *window) 
-{
-	galeon_window_show_find (window);
-}
-
 static void 
 print_dialog_preview_cb (PrintDialog *dialog, 
 			 GaleonWindow *window)
@@ -93,30 +94,6 @@ window_cmd_file_print (GtkAction *action,
 	galeon_dialog_show (dialog);
 }
 
-void
-window_cmd_go_back (GtkAction *action, 
-		    GaleonWindow *window) 
-{
-	GaleonEmbed *embed;
-
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	galeon_embed_go_back (embed);
-}
-
-void
-window_cmd_go_up (GtkAction *action, 
-		    GaleonWindow *window) 
-{
-	GaleonEmbed *embed;
-
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	galeon_embed_go_up (embed);
-}
-
 void 
 window_cmd_file_send_to	(GtkAction *action, 
 		      	 GaleonWindow *window) 
@@ -134,7 +111,9 @@ window_cmd_file_send_to	(GtkAction *action,
 	g_return_if_fail (tab);
 	
 	location = gnome_vfs_escape_string (galeon_tab_get_location (tab));
-	if (galeon_embed_get_title (embed, &title) == G_OK)
+	title = galeon_embed_get_title (embed);
+
+	if (title)
 	{
 		char *tmp = gnome_vfs_escape_string (title);
 		g_free (title);
@@ -156,36 +135,6 @@ window_cmd_file_send_to	(GtkAction *action,
 	g_free (url);
 }
 
-void
-window_cmd_go_forward (GtkAction *action, 
-		       GaleonWindow *window) 
-{
-	GaleonEmbed *embed;
-
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	galeon_embed_go_forward (embed);
-}
-
-void
-window_cmd_go_home (GtkAction *action, 
-		    GaleonWindow *window) 
-{
-	GaleonEmbed *embed;
-	char *location;
-	
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	location = eel_gconf_get_string (CONF_GENERAL_HOMEPAGE);
-	g_return_if_fail (location != NULL);
-	
-	galeon_embed_load_url (embed, location);
-
-	g_free (location);
-}
-
 void
 window_cmd_go_myportal (GtkAction *action, 
 		        GaleonWindow *window) 
@@ -251,11 +200,7 @@ open_new_tab (GaleonWindow *window, GaleonNewTabFlags flags)
 
 	tab = galeon_window_get_active_tab (window);
 	
-	tab = galeon_shell_new_tab (galeon_shell, window, tab, NULL, flags);
-
-	window = galeon_tab_get_window (tab);
-	galeon_window_set_location_entry_location (window, "");
-	galeon_window_edit_location (window);
+	galeon_shell_new_tab (galeon_shell, window, tab, NULL, flags);
 }
 
 void
@@ -414,10 +359,11 @@ window_cmd_file_save_as (GtkAction *action,
 	galeon_embed_persist_set_flags (persist,
 					EMBED_PERSIST_MAINDOC | 
 					EMBED_PERSIST_ASK_DESTINATION |
-					EMBED_PERSIST_SAVE_CONTENT);
-
+					EMBED_PERSIST_SAVE_CONTENT |
+					EMBED_PERSIST_ADD_TO_RECENT);
 	galeon_embed_persist_set_fc_parent (persist, GTK_WIDGET (window));
 	galeon_embed_persist_set_fc_title (persist, _("Save As..."));
+	galeon_embed_persist_set_user_time (persist, gtk_get_current_event_time());
 	galeon_embed_persist_save (persist);
 
 	g_object_unref (G_OBJECT(persist));
@@ -537,28 +483,35 @@ window_cmd_edit_select_all (GtkAction *action,
 	}
 }
 
+void
+window_cmd_edit_find (GtkAction *action, 
+		      GaleonWindow *window) 
+{
+	GaleonFindToolbar *toolbar;
+
+	toolbar = GALEON_FIND_TOOLBAR (galeon_window_get_find_toolbar (window));
+	galeon_find_toolbar_open (toolbar, FALSE, FALSE);
+}
+
+
 void
 window_cmd_edit_find_next (GtkAction *action, 
 			   GaleonWindow *window) 
 {
-	GaleonEmbed *embed;
+	GaleonFindToolbar *toolbar;
 
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	galeon_embed_find_next (embed, FALSE);
+	toolbar = GALEON_FIND_TOOLBAR (galeon_window_get_find_toolbar (window));
+	galeon_find_toolbar_find_next (toolbar);
 }
 
 void
 window_cmd_edit_find_prev (GtkAction *action, 
 			   GaleonWindow *window) 
 {
-	GaleonEmbed *embed;
+	GaleonFindToolbar *toolbar;
 
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	galeon_embed_find_next (embed, TRUE);
+	toolbar = GALEON_FIND_TOOLBAR (galeon_window_get_find_toolbar (window));
+	galeon_find_toolbar_find_previous (toolbar);
 }
 
 void 
@@ -566,8 +519,6 @@ window_cmd_set_permission(GtkAction *action,
 		      	  GaleonWindow *window) 
 {
 
-	gresult result;
-
 	gchar *url;
 	const char *verbname;
 	PermissionType type;
@@ -578,10 +529,10 @@ window_cmd_set_permission(GtkAction *action,
 
 	embed_shell = galeon_shell_get_embed_shell(galeon_shell);
 	g_return_if_fail(embed_shell != NULL);
-
+	
 	embed = galeon_window_get_active_embed(window);
-	result = galeon_embed_get_location(embed, TRUE, FALSE, &url);
-	if(result != G_OK) return;
+	url = galeon_embed_get_location(embed, TRUE, FALSE);
+	if (!url) return;
 
 	verbname = gtk_action_get_name (action);
 
@@ -618,24 +569,17 @@ window_cmd_show_java_console(GtkAction *action,
 			     GaleonWindow *window)
 {
 	GaleonEmbedShell *embed_shell;
-	gresult result;
 
 	embed_shell = galeon_shell_get_embed_shell(galeon_shell);
 	g_return_if_fail(embed_shell != NULL);
 
-	result = galeon_embed_shell_show_java_console(embed_shell);
+	galeon_embed_shell_show_java_console(embed_shell);
 }
 
 void window_cmd_show_js_console(GtkAction *action,
 				GaleonWindow *window)
 {
-	GaleonEmbedShell *embed_shell;
-	gresult result;
-
-	embed_shell = galeon_shell_get_embed_shell(galeon_shell);
-	g_return_if_fail(embed_shell != NULL);
-
-	result = galeon_embed_shell_show_js_console(embed_shell);
+	galeon_window_show_js_console (window);
 }
 
 void 
@@ -648,7 +592,7 @@ window_cmd_view_zoom_in	(GtkAction *action,
 	embed = galeon_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	galeon_embed_zoom_get (embed, &zoom);
+	zoom = galeon_embed_zoom_get (embed);
 	galeon_window_set_zoom (window, zoom + 10);
 }
 
@@ -662,7 +606,7 @@ window_cmd_view_zoom_out (GtkAction *action,
 	embed = galeon_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	galeon_embed_zoom_get (embed, &zoom);
+	zoom = galeon_embed_zoom_get (embed);
 	if (zoom >= 10)
 	{
 		galeon_window_set_zoom (window, zoom - 10);
@@ -687,6 +631,7 @@ window_cmd_view_page_source (GtkAction *action,
 
 	galeon_shell_new_tab (galeon_shell, window, tab, NULL, 
 			      GALEON_NEW_TAB_VIEW_SOURCE |
+			      GALEON_NEW_TAB_COPY_HISTORY |
 			      GALEON_NEW_TAB_JUMP);
 }
 
@@ -750,25 +695,31 @@ on_prefs_dialog_request (GaleonPrefsDialog  *dialog,
 		break;
 	case GALEON_PREFS_MANAGE_CERTIFICATES:
 	{
-		GaleonWindow *new_win;
-		GaleonTab    *tab;
+		GaleonSimpleWindow *new_win;
 
-		new_win = galeon_window_new ();
+		new_win = galeon_simple_window_new ();
 		gtk_window_resize (GTK_WINDOW(new_win), 550, 400);
-		galeon_window_set_chrome (new_win, EMBED_CHROME_CENTERSCREEN
-				                 | EMBED_CHROME_WINDOWRAISED
-						 | EMBED_CHROME_OPENASCHROME
-						 | EMBED_CHROME_OPENASPOPUP);
-		tab = galeon_tab_new ();
-		galeon_window_add_tab (new_win, tab,
-				       GALEON_WINDOW_ADD_TAB_LAST, TRUE);
-		
-		gtk_widget_show_all (GTK_WIDGET (tab));
 
-		galeon_window_load_url (new_win, "chrome://pippki/content/certManager.xul");
+		galeon_simple_window_load_url (new_win, "chrome://pippki/content/certManager.xul");
 
-		gtk_widget_show (GTK_WIDGET(new_win));
 		gtk_window_set_title (GTK_WINDOW(new_win), _("Certificate Manager"));
+		gtk_window_set_role (GTK_WINDOW(new_win), "galeon-certificate-manager");
+		gtk_widget_show (GTK_WIDGET(new_win));
+
+		break;
+	}
+	case GALEON_PREFS_MANAGE_DEVICES:
+	{
+		GaleonSimpleWindow *new_win;
+
+		new_win = galeon_simple_window_new ();
+		gtk_window_resize (GTK_WINDOW(new_win), 720, 250);
+
+		galeon_simple_window_load_url (new_win, "chrome://pippki/content/device_manager.xul");
+		
+		gtk_window_set_title (GTK_WINDOW(new_win), _("Device Manager"));
+		gtk_window_set_role (GTK_WINDOW(new_win), "galeon-device-manager");
+		gtk_widget_show (GTK_WIDGET(new_win));
 
 		break;
 	}
@@ -956,7 +907,7 @@ window_cmd_help_about (GtkAction *action,
 	};
 
 	/* Translator credits */
-	gchar *translator_credits = _("translator_credits");
+	gchar *translator_credits = _("translator-credits");
 
 	if (about != NULL)
 	{
@@ -974,20 +925,37 @@ window_cmd_help_about (GtkAction *action,
 	}
 	g_free (file);
 	
+#ifdef USE_GTKABOUTDIALOG
+	about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
+			      "name", _("Galeon"),
+			      "version", VERSION,
+			      "copyright", "Copyright \xc2\xa9 2003-2006 The Galeon Developers\nCopyright \xc2\xa9 2000-2002 Marco Pesenti Gritti",
+			      "authors", authors,
+			      "documenters", *documenters ? documenters : NULL,
+			      "comments", _("A GNOME browser based on Mozilla"),
+			      "translator-credits", translator_credits,
+			      "logo", pixbuf,
+			      NULL);
+
+	g_signal_connect (about, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+
+#else
 	about = gnome_about_new(
 		       _("Galeon"), VERSION,
-		       "Copyright \xc2\xa9 2003-2004 The Galeon Developers\nCopyright \xc2\xa9 2000-2002 Marco Pesenti Gritti",
+		       "Copyright \xc2\xa9 2003-2006 The Galeon Developers\nCopyright \xc2\xa9 2000-2002 Marco Pesenti Gritti",
 		       _("A GNOME browser based on Mozilla"),
 		       (const char **)authors,
 		       (const char **)documenters,
-		       strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+		       strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
 		       pixbuf);
+#endif
 	if (pixbuf) {
 		g_object_unref (G_OBJECT(pixbuf));
 	}			
 				
 	gtk_window_set_transient_for (GTK_WINDOW (about),
 				      GTK_WINDOW (window));
+	gtk_window_set_role (GTK_WINDOW (about), "galeon-about");
 	g_object_add_weak_pointer (G_OBJECT (about), (gpointer)&about);
 	gtk_widget_show (about);
 }
@@ -1063,6 +1031,7 @@ window_cmd_tabs_clone(GtkAction *action,
 	galeon_shell_new_tab(galeon_shell, window, tab, NULL,
 			     GALEON_NEW_TAB_IN_EXISTING_WINDOW |
 			     GALEON_NEW_TAB_JUMP |
+			     GALEON_NEW_TAB_COPY_HISTORY |
 			     GALEON_NEW_TAB_IS_A_COPY);
 }
 
