@@ -22,7 +22,6 @@
 
 #include <glib/gi18n.h>
 #include "galeon-autocompletion-window.h"
-#include "gul-gobject-misc.h"
 #include "gul-string.h"
 #include "galeon-marshal.h"
 #include "eel-gconf-extensions.h"
@@ -58,6 +57,10 @@
 /**
  * Private data
  */
+#define GALEON_AUTOCOMPLETION_WINDOW_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GALEON_TYPE_AUTOCOMPLETION_WINDOW, GaleonAutocompletionWindowPrivate))
+
+
 struct _GaleonAutocompletionWindowPrivate {
 	GaleonAutocompletion *autocompletion;
 	GtkWidget *parent;
@@ -80,8 +83,6 @@ struct _GaleonAutocompletionWindowPrivate {
 /**
  * Private functions, only availble from this file
  */
-static void	galeon_autocompletion_window_class_init		(GaleonAutocompletionWindowClass *klass);
-static void	galeon_autocompletion_window_init		(GaleonAutocompletionWindow *aw);
 static void	galeon_autocompletion_window_finalize_impl	(GObject *o);
 static void	galeon_autocompletion_window_init_widgets	(GaleonAutocompletionWindow *aw);
 static void	galeon_autocompletion_window_selection_changed_cb (GtkTreeSelection *treeselection, 
@@ -106,7 +107,6 @@ static void	galeon_autocompletion_window_autocompletion_sources_changed_cb (Gale
 
 
 
-static gpointer g_object_class;
 
 enum GaleonAutocompletionWindowSignalsEnum {
 	GALEON_AUTOCOMPLETION_WINDOW_URL_ACTIVATED,
@@ -119,15 +119,12 @@ static gint GaleonAutocompletionWindowSignals[GALEON_AUTOCOMPLETION_WINDOW_LAST_
  * AutocompletionWindow object
  */
 
-MAKE_GET_TYPE (galeon_autocompletion_window, "GaleonAutocompletionWindow", GaleonAutocompletionWindow, 
-	       galeon_autocompletion_window_class_init, 
-	       galeon_autocompletion_window_init, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GaleonAutocompletionWindow, galeon_autocompletion_window, G_TYPE_OBJECT);
 
 static void
 galeon_autocompletion_window_class_init (GaleonAutocompletionWindowClass *klass)
 {
 	G_OBJECT_CLASS (klass)->finalize = galeon_autocompletion_window_finalize_impl;
-	g_object_class = g_type_class_peek_parent (klass);
 	
 	GaleonAutocompletionWindowSignals[GALEON_AUTOCOMPLETION_WINDOW_URL_ACTIVATED] = g_signal_new (
 		"url-activated", G_OBJECT_CLASS_TYPE (klass),  
@@ -144,12 +141,14 @@ galeon_autocompletion_window_class_init (GaleonAutocompletionWindowClass *klass)
 		NULL, NULL, 
 		galeon_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
+
+	g_type_class_add_private (klass, sizeof (GaleonAutocompletionWindowPrivate));
 }
 
 static void 
 galeon_autocompletion_window_init (GaleonAutocompletionWindow *aw)
 {
-	GaleonAutocompletionWindowPrivate *p = g_new0 (GaleonAutocompletionWindowPrivate, 1);
+	GaleonAutocompletionWindowPrivate *p = GALEON_AUTOCOMPLETION_WINDOW_GET_PRIVATE (aw);
 	GtkTreeSelection *s;
 	
 	aw->priv = p;
@@ -200,12 +199,11 @@ galeon_autocompletion_window_finalize_impl (GObject *o)
 	
 	g_slist_foreach (p->selected, (GFunc) g_free, NULL);
 	g_slist_free (p->selected);
-	g_free (p);
 	
 	gdk_pointer_ungrab (GDK_CURRENT_TIME);
 	gdk_keyboard_ungrab (GDK_CURRENT_TIME);
 	
-	G_OBJECT_CLASS (g_object_class)->finalize (o);
+	G_OBJECT_CLASS (galeon_autocompletion_window_parent_class)->finalize (o);
 }
 
 static void
