@@ -107,6 +107,10 @@ typedef struct
 	PropertyType type;
 } PropertyInfo;
 
+#define GALEON_DIALOG_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GALEON_TYPE_DIALOG, GaleonDialogPrivate))
+
+
 struct GaleonDialogPrivate
 {
 	GtkWidget *parent;
@@ -186,6 +190,8 @@ galeon_dialog_class_init (GaleonDialogClass *klass)
                                                                "Modal dialog",
                                                                FALSE,
                                                                G_PARAM_READWRITE));
+
+	g_type_class_add_private (klass, sizeof (GaleonDialogPrivate));
 }
 
 static PrefType
@@ -459,7 +465,7 @@ prefs_connect_signals (GaleonDialog *dialog)
 static void
 galeon_dialog_init (GaleonDialog *dialog)
 {
-        dialog->priv = g_new0 (GaleonDialogPrivate, 1);
+        dialog->priv = GALEON_DIALOG_GET_PRIVATE (dialog);
 
 	dialog->priv->parent = NULL;
 	dialog->priv->dialog = NULL;
@@ -584,7 +590,7 @@ galeon_dialog_finalize (GObject *object)
         GaleonDialog *dialog;
 
         g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_GALEON_DIALOG (object));
+        g_return_if_fail (GALEON_IS_DIALOG (object));
 
         dialog = GALEON_DIALOG (object);
 
@@ -597,7 +603,6 @@ galeon_dialog_finalize (GObject *object)
 	
 	g_free (dialog->priv->name);
 	g_free (dialog->priv->props);
-        g_free (dialog->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -656,13 +661,13 @@ galeon_dialog_set_parent (GaleonDialog *dialog,
 GaleonDialog *
 galeon_dialog_new (void)
 {
-	return GALEON_DIALOG (g_object_new (GALEON_DIALOG_TYPE, NULL));
+	return GALEON_DIALOG (g_object_new (GALEON_TYPE_DIALOG, NULL));
 }
 
 GaleonDialog *
 galeon_dialog_new_with_parent (GtkWidget *parent_window)
 {
-	return GALEON_DIALOG (g_object_new (GALEON_DIALOG_TYPE,
+	return GALEON_DIALOG (g_object_new (GALEON_TYPE_DIALOG,
 					    "ParentWindow", parent_window,
 					    NULL));
 }
@@ -890,7 +895,7 @@ galeon_dialog_construct (GaleonDialog *dialog,
 			 const char *name)
 {
 	GaleonDialogClass *klass = GALEON_DIALOG_GET_CLASS (dialog);
-        return klass->construct (dialog, properties, file, name);
+        klass->construct (dialog, properties, file, name);
 }
 
 void 
@@ -928,6 +933,6 @@ galeon_dialog_get_value (GaleonDialog *dialog,
 			 GValue *value)
 {
 	GaleonDialogClass *klass = GALEON_DIALOG_GET_CLASS (dialog);
-        return klass->get_value (dialog, property_id, value);
+        klass->get_value (dialog, property_id, value);
 }
 
