@@ -15,30 +15,41 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+/* Allow deprecated for the gnome_entry widget */
+#undef GTK_DISABLE_DEPRECATED
+#undef GNOME_DISABLE_DEPRECATED
 
 #include "location-entry.h"
 #include "galeon-autocompletion-window.h"
 #include "galeon-marshal.h"
-#include "gul-gobject-misc.h"
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
+#include "galeon-debug.h"
 
-#include <gtk/gtkentry.h>
-#include <gtk/gtkwindow.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkmain.h>
 #include <gtk/gtkclipboard.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkimagemenuitem.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkseparatormenuitem.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkwindow.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-entry.h>
 #include <string.h>
 
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
-
-#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-
 /**
  * Private data
  */
+#define GALEON_LOCATION_ENTRY_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GALEON_TYPE_LOCATION_ENTRY, GaleonLocationEntryPrivate))
+
+
 struct _GaleonLocationEntryPrivate {
 	GtkWidget *combo;
 	GtkWidget *entry;
@@ -54,6 +65,8 @@ struct _GaleonLocationEntryPrivate {
 
 	gchar *autocompletion_key;
 	gchar *last_completion;
+
+	GtkWidget *shown_widget; /* for auto-toolbar show */
 };
 
 #define AUTOCOMPLETION_DELAY 200
@@ -64,8 +77,6 @@ struct _GaleonLocationEntryPrivate {
 /**
  * Private functions, only availble from this file
  */
-static void		galeon_location_entry_class_init	(GaleonLocationEntryClass *klass);
-static void		galeon_location_entry_init		(GaleonLocationEntry *w);
 static void		galeon_location_entry_finalize_impl	(GObject *o);
 static void		galeon_location_entry_build		(GaleonLocationEntry *w);
 static gboolean		galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
@@ -94,7 +105,6 @@ static void		galeon_location_entry_autocompletion_window_hidden_cb (GaleonAutoco
 
 
 
-static gpointer gtk_hbox_class;
 
 /**
  * Signals enums and ids
@@ -109,15 +119,14 @@ static gint GaleonLocationEntrySignals[GALEON_LOCATION_ENTRY_LAST_SIGNAL];
  * GaleonLocationEntry object
  */
 
-MAKE_GET_TYPE (galeon_location_entry, "GaleonLocationEntry", GaleonLocationEntry, 
-	       galeon_location_entry_class_init, 
-	       galeon_location_entry_init, GTK_TYPE_HBOX);
+G_DEFINE_TYPE (GaleonLocationEntry, galeon_location_entry, GTK_TYPE_HBOX);
 
 static void
 galeon_location_entry_class_init (GaleonLocationEntryClass *klass)
 {
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
+
 	G_OBJECT_CLASS (klass)->finalize = galeon_location_entry_finalize_impl;
-	gtk_hbox_class = g_type_class_peek_parent (klass);
 
 	GaleonLocationEntrySignals[GALEON_LOCATION_ENTRY_URL_ACTIVATED] = g_signal_new (
 		"url-activated", G_OBJECT_CLASS_TYPE (klass),  
@@ -126,6 +135,25 @@ galeon_location_entry_class_init (GaleonLocationEntryClass *klass)
 		NULL, NULL, 
 		galeon_marshal_VOID__STRING_INT,
 		G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_BOOLEAN);
+
+	gtk_widget_class_install_style_property (widget_class,
+						 g_param_spec_boxed ("secure-color",
+								     "Secure color",
+								     "Background color to use for secure sites",
+								     GDK_TYPE_COLOR,
+								     G_PARAM_READABLE)); 
+
+	g_type_class_add_private (klass, sizeof (GaleonLocationEntryPrivate));
+}
+
+static void
+galeon_location_entry_activation_finished (GaleonLocationEntry *entry)
+{
+	if (entry->priv->shown_widget)
+	{
+		gtk_widget_hide (entry->priv->shown_widget);
+		entry->priv->shown_widget = NULL;
+	}
 }
 
 static gboolean
@@ -133,17 +161,54 @@ focus_out_event_cb (GtkWidget *entry, GdkEventFocus *event,
 		    GaleonLocationEntry *w)
 {
 	w->priv->editing = FALSE;
+	galeon_location_entry_activation_finished (w);
 	return FALSE;
 }
 
+static void
+entry_clear_activate_cb (GtkMenuItem *item, GtkEntry *entry)
+{
+	gtk_entry_set_text (entry, "");
+}
+
+static void
+entry_populate_popup_cb (GtkEntry *entry, GtkMenu *menu)
+{
+	GtkWidget *image;
+	GtkWidget *menuitem;
+
+	menuitem = gtk_separator_menu_item_new ();
+	gtk_widget_show (menuitem);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL(menu), menuitem);
+
+	/* Clear and Copy mnemonics conflict, make custom menuitem */
+	image = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
+	/* To translators: the mnemonic shouldn't conflict with any of the
+	 * stock items in the GtkEntry context menu (Cut, Copy, Paste, Delete,
+	 * and Insert Unicode control character.) */
+	menuitem = gtk_image_menu_item_new_with_mnemonic (_("Cl_ear"));
+
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(menuitem), image);
+
+	g_signal_connect (G_OBJECT(menuitem), "activate",
+			  G_CALLBACK(entry_clear_activate_cb), entry);
+
+	gtk_widget_show (image);
+	gtk_widget_show (menuitem);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL(menu), menuitem);
+}
+
 static void 
 galeon_location_entry_init (GaleonLocationEntry *w)
 {
-	GaleonLocationEntryPrivate *p = g_new0 (GaleonLocationEntryPrivate, 1);
+	GaleonLocationEntryPrivate *p = GALEON_LOCATION_ENTRY_GET_PRIVATE (w);
 	w->priv = p;
 
 	galeon_location_entry_build (w);
 
+	g_signal_connect (w->priv->entry, "populate_popup",
+			  G_CALLBACK (entry_populate_popup_cb), NULL);
+
 	g_signal_connect (w->priv->entry, "focus_out_event",
 			  G_CALLBACK (focus_out_event_cb), w);
 }
@@ -166,11 +231,12 @@ galeon_location_entry_finalize_impl (GObject *o)
 		g_object_unref (G_OBJECT (p->autocompletion_window)); 
 	}
 
+	g_free (p->last_completion);
+	g_free (p->before_completion);
 	g_free (p->autocompletion_key);
-	DEBUG_MSG (("GaleonLocationEntry finalized\n"));
+	LOG ("GaleonLocationEntry finalized");
 
-	g_free (p);	
-	G_OBJECT_CLASS (gtk_hbox_class)->finalize (o);
+	G_OBJECT_CLASS (galeon_location_entry_parent_class)->finalize (o);
 }
 
 GaleonLocationEntry *
@@ -222,7 +288,7 @@ galeon_location_entry_autocompletion_show_alternatives_to (GaleonLocationEntry *
 	GaleonLocationEntryPrivate *p = w->priv;
 	if (p->autocompletion)
 	{
-		DEBUG_MSG (("+galeon_location_entry_autocompletion_show_alternatives_to\n"));
+		LOG ("+galeon_location_entry_autocompletion_show_alternatives_to");
 		galeon_location_entry_set_autocompletion_key (w);
 		galeon_location_entry_autocompletion_show_alternatives (w);
 	}
@@ -269,7 +335,7 @@ galeon_location_entry_autocompletion_to (GaleonLocationEntry *w)
 	gchar *text;
 	gchar *common_prefix;
 	
-	DEBUG_MSG (("in galeon_location_entry_autocompletion_to\n"));
+	LOG ("in galeon_location_entry_autocompletion_to");
 
 	galeon_location_entry_set_autocompletion_key (w);
 
@@ -285,8 +351,9 @@ galeon_location_entry_autocompletion_to (GaleonLocationEntry *w)
 		    || send != text_len)
 		{
 			/* the user is editing the entry, don't mess it */
-			DEBUG_MSG (("The user seems editing the text: pos = %d, strlen (text) = %d, sstart = %d, send = %d\n", 
-				    pos, strlen (text), sstart, send));
+			LOG ("The user seems editing the text: pos = %d, "
+			     "strlen (text) = %d, sstart = %d, send = %d", 
+			     pos, strlen (text), sstart, send);
 			p->autocompletion_timeout = 0;
 			return FALSE;
 		}		
@@ -294,7 +361,7 @@ galeon_location_entry_autocompletion_to (GaleonLocationEntry *w)
 
 	common_prefix = galeon_autocompletion_get_common_prefix (p->autocompletion);
 
-	DEBUG_MSG (("common_prefix: %s\n", common_prefix));
+	LOG ("common_prefix: %s", common_prefix);
 
 	if (common_prefix && (!p->before_completion || p->before_completion[0] == '\0'))
 	{
@@ -398,7 +465,7 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
 	{
 		char *url = gtk_editable_get_chars (GTK_EDITABLE(entry), 0, -1);
 
-		DEBUG_MSG (("In galeon_location_key_press_cb, going to %s\n", url));
+		LOG ("In galeon_location_key_press_cb, going to %s", url);
 		   
 		galeon_location_entry_autocompletion_hide_alternatives (w);
 
@@ -448,7 +515,7 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
         case GDK_Page_Up:
         case GDK_Page_Down:
 		galeon_location_entry_autocompletion_hide_alternatives (w);
-                //galeon_embed_grab_focus (window->active_embed);
+                /* galeon_embed_grab_focus (window->active_embed); */
                 return FALSE;
         case GDK_Tab:
         {
@@ -461,9 +528,11 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
                 text = galeon_location_entry_get_location (w);
 		galeon_location_entry_autocompletion_unselect_alternatives (w);
 
-		// TODO: heuristic completions like in galeon1 are missing
-                //common_prefix = 
-		//      auto_completion_complete_url_extended (text, suggest);
+		/*
+		 * TODO: heuristic completions like in galeon1 are missing
+		 * common_prefix = 
+		 *     auto_completion_complete_url_extended (text, suggest);
+		 */
 
 		if (p->autocompletion)
 		{
@@ -521,7 +590,7 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
                 }
                 else
                 {
-                        //real_set_location (w, embed->location);
+                        /* real_set_location (w, embed->location); */
                 }
                 break;
         default:
@@ -560,7 +629,7 @@ galeon_location_entry_activate_cb (GtkEntry *entry, GaleonLocationEntry *w)
 
 	galeon_location_entry_autocompletion_hide_alternatives (w);
 
-	DEBUG_MSG (("In galeon_location_entry_activate_cb, going to %s\n", url));
+	LOG ("In galeon_location_entry_activate_cb, going to %s", url);
 
 	emit_activated_signal (w, url, FALSE);
 
@@ -573,7 +642,7 @@ galeon_location_entry_autocompletion_sources_changed_cb (GaleonAutocompletion *a
 {
 	GaleonLocationEntryPrivate *p = w->priv;
 
-	DEBUG_MSG (("in galeon_location_entry_autocompletion_sources_changed_cb\n"));
+	LOG ("in galeon_location_entry_autocompletion_sources_changed_cb");
 
         if (p->autocompletion_timeout == 0
 	    && p->last_completion
@@ -657,7 +726,7 @@ galeon_location_entry_autocompletion_window_url_activated_cb (GaleonAutocompleti
 {
 	real_set_location (w, url);
 
-	DEBUG_MSG (("In location_entry_autocompletion_window_url_activated_cb, going to %s\n", url));
+	LOG ("In location_entry_autocompletion_window_url_activated_cb, going to %s", url);
 
 	galeon_location_entry_autocompletion_hide_alternatives (w);
 
@@ -670,7 +739,7 @@ galeon_location_entry_autocompletion_window_hidden_cb (GaleonAutocompletionWindo
 {
 	GaleonLocationEntryPrivate *p = w->priv;
 
-	DEBUG_MSG (("In location_entry_autocompletion_window_hidden_cb\n"));
+	LOG ("In location_entry_autocompletion_window_hidden_cb");
 
 	p->autocompletion_window_visible = FALSE;
 
@@ -690,6 +759,21 @@ galeon_location_entry_autocompletion_window_hidden_cb (GaleonAutocompletionWindo
 void
 galeon_location_entry_edit (GaleonLocationEntry *w)
 {
+	GtkWidget *widget;
+
+	/* Traverse up the widget hierarchy to see if a parent
+	 * widget is hidden, and if it is, show it, this lets
+	 * that toolbar popup when the user uses Ctrl+L */
+	for (widget = GTK_WIDGET (w); widget != NULL; widget = widget->parent)
+	{
+		if (!GTK_WIDGET_VISIBLE (widget))
+		{
+			w->priv->shown_widget = widget;
+			gtk_widget_show (w->priv->shown_widget);
+			break;
+		}
+	}
+
 	w->priv->editing = TRUE;
 	/* gtk_editable_select_region (GTK_EDITABLE(w->priv->entry), 0, -1); */
 	gtk_widget_grab_focus (w->priv->entry);
@@ -744,3 +828,25 @@ galeon_location_entry_set_autocompletion_key (GaleonLocationEntry *e)
 	}
 }
 
+void
+galeon_location_entry_set_is_secure (GaleonLocationEntry *entry, gboolean is_secure)
+{
+	GaleonLocationEntryPrivate *p = entry->priv;
+
+	if (is_secure)
+	{
+		static const GdkColor fallback = { 0, 0xf7f7, 0xf7f7, 0xbebe };
+		GdkColor             *secure_color = NULL;
+
+		gtk_widget_style_get (GTK_WIDGET (entry), "secure-color", &secure_color, NULL);
+		if (secure_color == NULL)
+			secure_color = (GdkColor*)&fallback;
+
+		gtk_widget_modify_base (p->entry, GTK_STATE_NORMAL, secure_color);
+		
+		if (secure_color != &fallback)
+			gdk_color_free (secure_color);
+	}
+	else
+		gtk_widget_modify_base (p->entry, GTK_STATE_NORMAL, NULL);
+}
